import asyncio
from typing import Any, Callable, Optional, Union
from PySide6.QtWidgets import QWidget, QVBoxLayout, QFrame
from PySide6.QtCore import Qt, Signal, Slot, Property, QTimer

from qtmui.hooks import useEffect
from qtmui.hooks import State

from ..button import Button, LoadingButton
from .rhf_submit import SubmitButton
from .rhf_text_field import RHFTextField
from .rhf_autocomplete import RHFAutocomplete
from .rhf_select import RHFSelect
from .rhf_radio_group import RHFRadioGroup
from .rhf_switch import RHFSwitch
from .rhf_rating import RHFRating
from .rhf_checkbox import RHFCheckbox, RHFMultiCheckbox
from .rhf_slider import RHFSlider
from .rhf_upload import RHFUploadAvatar

from .types import ResolverType, UseFormType



class FormProvider(QFrame):
    formState = Signal(dict)
    submitSignal = Signal(object)

    def __init__(
            self, 
            onSubmit: Callable = None, 
            initForm: Optional[State] = None, 
            schema: object = None, 
            children: object = None, 
            methods: UseFormType = None
        ):
        super().__init__()
        self.validator = methods.resolver
        self.methods: UseFormType = methods
        
        self.setLayout(QVBoxLayout())
        self.layout().setAlignment(Qt.AlignmentFlag.AlignTop)
        self.layout().setContentsMargins(0, 0, 0, 0)
        self.data = {}
        self.none_value = {}
        self.schema = schema
        self.submitCount = 0
        self.submitSignal.connect(onSubmit)
        
        self.initForm = initForm # dành cho các form lớn
        
        self.isReseting = False
        
        if isinstance(children, list):
            for item in children:
                if item is not None:
                    self.layout().addWidget(item)
                    
        if isinstance(initForm, State):
            initForm.valueChanged.connect(self._setup_form)
        else:
            self._setup_form()

        self.methods.state.valueChanged.connect(self.set_form_data)
        
        useEffect(
            self._reset,
            [self.methods.isReset]
        )
        
        self.destroyed.connect(self._onDestroy)

    def _setup_form(self):
        if isinstance(self.initForm, State) and self.initForm.value:
            QTimer.singleShot(0, self._schedule_setup_form)
            QTimer.singleShot(0, self._schedule_set_form_data)
        else:
            QTimer.singleShot(0, self._schedule_setup_form)
            QTimer.singleShot(0, self._schedule_set_form_data)

    def _onDestroy(self, obj=None):
        # Cancel task nếu đang chạy
        self.methods.state.valueChanged.disconnect(self.set_form_data)
        if self._setup_form_task and not self._setup_form_task.done():
            self._setup_form_task.cancel( )
        if self._set_form_data_task and not self._set_form_data_task.done():
            self._set_form_data_task.cancel()
            
    def _schedule_setup_form(self):
        self._setup_form_task = asyncio.ensure_future(self.setup_form())
        
    def _schedule_set_form_data(self):
        self._set_form_data_task = asyncio.ensure_future(self.async_set_form_data(self.methods.state.value))
        
    def _reset(self):
        self.isReseting = True
        if self.methods.isReset.value:
            self.methods.setIsReset(False)
            QTimer.singleShot(0, lambda: asyncio.ensure_future(self.async_set_form_data(self.methods.state.value, reseting=True)))
        self.submitCount = 0
        
    async def setup_form(self):
    # def setup_form(self):
        rhf_fields = []
        for control in self.findChildren(QWidget):
            if type(control) == RHFTextField:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFAutocomplete:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFSelect:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFSwitch:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFRating:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFCheckbox:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFMultiCheckbox:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFRadioGroup:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif type(control) == RHFSlider:
                control.valueChanged.connect(lambda value=control._value, name=control._name: self.on_value_changed(name, value))
                control.stateSignal = self.formState
            elif isinstance(control, Button) or isinstance(control, SubmitButton):
                if control._type == "submit":
                    # print('submitiiiiiiiiiiiiiiiii')
                    control.clicked.connect(self._on_submit)
            
            if type(control) in [
                RHFTextField, 
                RHFSelect, 
                RHFAutocomplete, 
                RHFUploadAvatar,
                RHFCheckbox,
                RHFSwitch,
                RHFRating,
                RHFMultiCheckbox,
                RHFRadioGroup,
                RHFSlider,
                ]:
                rhf_fields.append(control)
        self.methods.setFields(rhf_fields)

    def _set_control_value(self, control, value):
        control.set_value(value=value)

    def set_form_data(self, data: Optional[Union[dict, Any]], reseting=False):
        
        for control in self.findChildren(QWidget):
            if type(control) in [
                RHFTextField, 
                RHFSelect, 
                RHFAutocomplete, 
                RHFUploadAvatar,
                RHFCheckbox,
                RHFSwitch,
                RHFRating,
                RHFMultiCheckbox,
                RHFRadioGroup,
                RHFSlider,
                ]:
                if hasattr(control, "_name"):
                    value = data.get(control._name)
                    # print(f"Setting value for control '{control._name}': {value}")
                if value is not None:
                    self._set_control_value(control, value)

        if reseting:
            self.isReseting = False
            
    async def async_set_form_data(self, data: Optional[Union[dict, Any]], reseting=False):
        
        for control in self.findChildren(QWidget):
            if type(control) in [
                RHFTextField, 
                RHFSelect, 
                RHFAutocomplete, 
                RHFUploadAvatar,
                RHFCheckbox,
                RHFSwitch,
                RHFRating,
                RHFMultiCheckbox,
                RHFRadioGroup,
                RHFSlider,
                ]:
                if hasattr(control, "_name"):
                    value = data.get(control._name)
                    # print(f"Setting value for control '{control._name}': {value}")
                if value is not None:
                    self._set_control_value(control, value)

        if reseting:
            self.isReseting = False

    def _on_reset(self):
        self.submitCount = 0

    def _on_submit(self):
        print('_on_submit')
        self.submitCount += 1
        if self.validate():
            self.submitSignal.emit(self.data)

    def get_form_data(self):
        for control in self.findChildren(QWidget):
            if str(type(control)).find('RHF') != -1:
                if control._value is not None:
                    self.data[control._name] = control._value
                else: 
                    if self.data.get(control._name) is not None:
                        del self.data[control._name]
        self.methods.setData(self.data)

    def validate(self, field=None):

        if not self.submitCount:
            # self.get_form_data() # chỗ này làm nặng chương trình
            return False
        self.get_form_data()
        schema = self.validator[0]
        v = self.validator[1]()

        if v.validate(self.data):
            if field:
                self.formState.emit({"field": field, "error": False})
            # self.submitSignal.emit(self.data)
            return True
        else:
            valid_fields = set(schema.keys()) - set(v.errors.keys())
            print(v.errors)
            for field in list(valid_fields):
                self.formState.emit({"field": field, "error": False})

            for field, error in v.errors.items():
                self.formState.emit({"field": field, "error": True, "error_message": error})
            return False

    def on_value_changed(self, name, value):
        if value == Qt.CheckState.Checked:
            value = True
        elif value == Qt.CheckState.Unchecked:
            value = False

        if value is not None:
            self.data[name] = value
        else:
            self.none_value[name] = value
            
        # print(f"Control '{name}' has value: {value}")
        if not self.isReseting:
            self.validate(name)

    # @Slot(object)
    # def set_state(self, state):
    #     for control in self.findChildren(QWidget):
    #         if type(control) == RHFTextField:
    #             if control._name == state.get('field'):
    #                 control.set_error(error)




from qtmui.material.styles.create_theme.theme_reducer import ThemeState
from ....system.color_manipulator import alpha

def app_bar(_theme):
    theme: ThemeState = _theme
    
    return {
        'PyBackdrop': {
            'styles': {
                'root': {
                    backgroundColor: alpha(theme.palette.grey._900, 0.8),
                }
            }
        }
    }



from PySide6.QtWidgets import QFrame, QHBoxLayout

from ..stack import Stack
from ..progress import CircularProgress, LinearProgress

class SplashScreen(QFrame):
    def __init__(
            self,
            variant: str = "circular", # linear
            color: str = "default"
    ):
        super().__init__()
        self.setLayout(QHBoxLayout())
        self.layout().addWidget(
            Stack(
                # hFlexCenter=True,
                # layoutAlignHCenter=True,
                children=[
                    CircularProgress(
                        key="color", 
                        color=color
                    ) if variant == "circular"
                    else LinearProgress(
                        key="color", 
                        color=color    
                    )
                ]
            )
        )




import inspect
from typing import Callable, Dict, Union, Optional
from .qss_properties import QssProperties
from qtmui.material.styles import useTheme
from functools import lru_cache


def camel_to_kebab_case(s: str) -> str:
    """
    Chuyển đổi chuỗi từ camelCase sang kebab-case.
    """
    return ''.join(['-' + c.lower() if c.isupper() else c for c in s]).lstrip('-')


def resolve_theme_value(theme, value: Union[str, Callable]) -> Optional[str]:
    """
    Giải quyết giá trị theme từ chuỗi dạng 'palette.primary.main', 'text.secondary', v.v.
    """
    if isinstance(value, str) and (
        value.startswith("text.") 
        or value.startswith("background.")
        or value.startswith("primary.")
        or value.startswith("secondary.")
        or value.startswith("info.")
        or value.startswith("warning.")
        or value.startswith("error.")
        ):
        value = f"palette.{value}"

    if isinstance(value, str) and value.startswith("palette."):
        parts = value.split(".")
        theme_value = theme
        _value = None
        for part in parts:
            theme_value = getattr(theme_value, part, None)
            if theme_value is None:
                return None

        return theme_value
    elif isinstance(value, Callable):
        # return value(theme) # cũ 
        
        sig = inspect.signature(value)
        # Tạo đối số mặc định nếu không có
        default_args = {
            k: (v.default if v.default is not inspect.Parameter.empty else None)
            for k, v in sig.parameters.items()
        }

        try:
            return value(**default_args)
        except Exception as e:
            return None

    return value


def get_qss_style(sx: Dict, class_name: Optional[str] = None) -> str:
    """
    Hàm tạo chuỗi QSS từ dictionary sx, hỗ trợ nested selectors và pseudo-classes.
    Nếu class_name=None, chỉ trả về chuỗi thuộc tính mà không bao bọc trong block.
    """
    if isinstance(sx, str):
        return sx

    theme = useTheme()
    base_styles = []  # Styles cấp cao
    nested_styles = []  # Styles cho các selector lồng nhau

    for key, value in sx.items():
        # Chỉ áp dụng camel_to_kebab_case cho các key không bắt đầu bằng "&"
        if not key.startswith("&"):
            css_key = camel_to_kebab_case(key)
        else:
            css_key = key

        # Xử lý các selector lồng nhau (bắt đầu bằng "&")
        if css_key.startswith("&"):
            selector = css_key[1:].strip()  # Bỏ "&" và khoảng trắng
            if class_name is None:
                continue  # Bỏ qua nested selectors nếu không có class_name
            if selector.startswith(":"):  # Pseudo-class như &:hover
                pseudo_part, *child_part = selector.split(" ", 1)
                pseudo = pseudo_part[1:]  # Bỏ ":"
                if child_part:  # Có selector con, ví dụ &:hover .Typography
                    nested_selector = f"{class_name}[slot={pseudo}] {child_part[0].lstrip('.')}"
                else:
                    nested_selector = f"{class_name}[slot={pseudo}]"
            else:  # Không có pseudo-class, ví dụ & Typography hoặc & #container
                if " " in selector:  # Có khoảng trắng, ví dụ & Typography
                    nested_selector = selector.split(" ", 1)[1]  # Lấy phần sau khoảng trắng
                else:
                    nested_selector = f"{class_name} {selector}"

            if isinstance(value, dict):
                nested_qss = get_qss_style(value, nested_selector)
                nested_styles.append(nested_qss)
            continue

        # Xử lý thuộc tính CSS cấp cao
        
        resolved_value = resolve_theme_value(theme, value)
        if resolved_value is None:
            continue

        if css_key in QssProperties.__dict__.values():

            if css_key in ["marginX", "mx"]:
                base_styles.append(f"margin-left: {resolved_value};")
                base_styles.append(f"margin-right: {resolved_value};")
            elif css_key in ["marginY", "my"]:
                base_styles.append(f"margin-top: {resolved_value};")
                base_styles.append(f"margin-bottom: {resolved_value};")
            elif css_key in ["paddingX", "px"]:
                base_styles.append(f"padding-left: {resolved_value};")
                base_styles.append(f"padding-right: {resolved_value};")
            elif css_key in ["paddingY", "py"]:
                base_styles.append(f"padding-top: {resolved_value};")
                base_styles.append(f"padding-bottom: {resolved_value};")
            elif css_key == "width" and str(resolved_value).find("%") == -1:
                base_styles.append(f"min-width: {resolved_value};")
                base_styles.append(f"max-width: {resolved_value};")
            elif css_key == "height" and str(resolved_value).find("%") == -1:
                base_styles.append(f"min-height: {resolved_value};")
                base_styles.append(f"max-height: {resolved_value};")
            elif css_key not in ["mb", "display", "flex-shrink", "flex-grow", "align-items", 
                                "flex-direction", "align-self", "z-index"]:
                base_styles.append(f"{css_key}: {resolved_value};")

    # Tạo QSS
    qss = ""
    if base_styles:
        if class_name:
            qss = f"{class_name} {{\n    " + "\n    ".join(base_styles) + "\n}}"
        else:
            qss = "\n    ".join(base_styles)  # Chỉ trả về chuỗi thuộc tính nếu không có class_name
    if nested_styles:
        if qss:
            qss += "\n\n"
        qss += "\n\n".join(nested_styles)

    return qss.strip().replace("}}", "}")


from typing import Callable
from PySide6.QtWidgets import QWidget, QHBoxLayout, QVBoxLayout, QPushButton

from ...common.ui_functions import clear_layout
from qtmui.hooks import State



class WidgetView(QWidget):
    def __init__(
                self,
                direction="column",
                renderView: Callable = None,
                renderViewProps: Callable = None,
                children: State = None,
                view: State = None
                ):
        super().__init__()
        if direction == "column":
            self.setLayout(QHBoxLayout())
        else:
            self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        self._children = children
        if not self._children:
            self._children = view
        
        if isinstance(children, State):
            children.valueChanged.connect(self._render_ui)
            self._render_ui(children.value)

    def _render_ui(self, widget):
        if self.layout().count():
            clear_layout(self.layout())
        if widget:
            self.layout().addWidget(widget)

QPushButton




from PySide6.QtWidgets import QPushButton
from PySide6.QtCore import QSize, QCoreApplication, QEvent, Signal, QRect
from PySide6.QtGui import QMovie, QIcon, Qt, QColor
from PySide6.QtWidgets import QStyledItemDelegate, QStyle, QApplication, QStyleOptionButton, QPushButton

from ...utils.icon import icon_base64_to_pixmap

class StyledOptionButton(QPushButton):
    def __init__(
            self, 
            parent=None,
            name: str = None,
            iconBase64: object = None,
            size: QSize = QSize(50, 50),
            iconSize: QSize = QSize(15, 15)
            ):
        super(StyledOptionButton, self).__init__(parent)

        # self.setLayout(QHBoxLayout())
        self.setFixedSize(size)

        self.option = QStyleOptionButton()
        self.option.icon.addPixmap(icon_base64_to_pixmap(iconBase64), QIcon.Normal, QIcon.Off)
        self.option.iconSize = iconSize

        # rgb(227, 227, 227)
        
        self.setStyleSheet("""
            QPushButton{
                background-color: rgb(237, 237, 237);
                max-height: 30px;
                min-height: 30px;
                border: 0px solid transparent;
                border-radius: 4px;
                margin-left: 2px;
                color: #6b6b6b;
            }
            QPushButton::hover {
                border: 2px solid red;
                background-color: red;
            }
            """)
        self.option.initFrom(self)




# Copyright (C) 2022 The Pyact Company Ltd.
# SPDX-License-Identifier: LicenseRef-Pyact-Commercial OR BSD-3-Clause
from __future__ import annotations

"""..site_packages.qtcompat port of the areachart example from qtmui v1.0"""

from typing import Optional, Union
import sys
from PySide6.QtCore import QPointF, Qt
from PySide6.QtWidgets import QMainWindow, QApplication, QWidget, QVBoxLayout

from .map_change_theme import MapChangeTheme

from .map_widget import MapWindow

class Map(QWidget):
    def __init__(
                self,
                initialViewState: dict = None, 
                *args,
                **kwargs
                ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        map = MapChangeTheme(
            initialViewState=initialViewState,
        )

        # map = MapWindow()
        self.layout().addWidget(map)



from PySide6.QtWidgets import QVBoxLayout, QWidget

class Layout(QWidget):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

    def add_widget(self, element: QWidget):
        """Thêm một phần tử giao diện vào trang."""
        if isinstance(element, QWidget):
            self.layout().addWidget(element)
        else:
            print(f"Opp!! element must have type QWidget")




# qtmui/material/chart/chart_radial_bar.py
from __future__ import annotations

from typing import Any, Dict, List, Optional, Union, Callable
from math import cos, radians, sin
from PySide6.QtCore import Qt, QPointF, QRectF, QTimer
from PySide6.QtWidgets import QWidget
from PySide6.QtGui import (
    QPainter, QColor, QBrush, QPen, QPainterPath, QFontMetrics
)

from qtmui.material.styles import useTheme


class ChartRadialBar(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        type: str = "radialBar",
        series: Optional[List[float]] = None,
        total: Optional[float] = None,
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: Optional[Dict[str, Any]] = None,
        key: Optional[str] = None,
        *args, **kwargs
    ):
        super().__init__(*args, **kwargs)
        self._series = series or []
        self._total = total  # nếu có thì dùng làm base 100%
        self._options = options or {}
        self._width = int(width) if width else None
        self._height = int(height) if height else None

        if self._width: self.setFixedWidth(self._width)
        if self._height: self.setFixedHeight(self._height)
        self.setMinimumSize(180, 180)

        self.theme = useTheme()
        self.theme.state.valueChanged.connect(self.update)

        self.labels = self._options.get("labels", [f"Series {i+1}" for i in range(len(self._series))])
        self.colors = self._options.get("colors", [
            self.theme.palette.primary.main,
            self.theme.palette.secondary.main,
            self.theme.palette.success.main,
            self.theme.palette.warning.main,
            self.theme.palette.error.main,
        ])

        self._prepare_options()

    def _prepare_options(self):
        po = self._options.get("plotOptions", {}).get("radialBar", {})

        # Hollow size
        hollow = po.get("hollow", {}).get("size", "65%")
        self.hollow_ratio = float(hollow.rstrip("%")) / 100 if isinstance(hollow, str) else 0.65

        # Track (nền)
        self.track_color = po.get("track", {}).get("background", "#e0e0e033")
        self.track_opacity = po.get("track", {}).get("opacity", 0.2)

        # Data labels
        self.data_labels = po.get("dataLabels", {})
        self.show_names = self.data_labels.get("name", {}).get("show", True)
        self.show_values = self.data_labels.get("value", {}).get("show", True)

        # Total
        total_opts = self.data_labels.get("total", {})
        self.show_total = total_opts.get("show", True)
        self.total_formatter = total_opts.get("formatter")

        # Start từ 12h (top), đi theo chiều kim đồng hồ
        self.start_angle = 270  # 12h
        self.total_angle = 360

        # Base value
        self.base_value = self._total or 100.0

    def paintEvent(self, event):
        p = QPainter(self)
        p.setRenderHint(QPainter.Antialiasing, True)

        rect = self.rect()
        side = min(rect.width(), rect.height())
        draw_rect = QRectF(
            rect.width() / 2 - side / 2 + 16,
            rect.height() / 2 - side / 2 + 16,
            side - 32,
            side - 32
        )
        center = draw_rect.center()
        max_radius = draw_rect.width() / 2

        inner_radius = max_radius * self.hollow_ratio
        bar_thickness = (max_radius - inner_radius) / max(1, len(self._series))

        # Vẽ từng thanh + track riêng
        current_angle = self.start_angle

        for idx, value in enumerate(self._series):
            percentage = value / self.base_value
            angle_span = self.total_angle * percentage

            outer_r = max_radius - idx * bar_thickness
            inner_r = outer_r - bar_thickness * 0.85
            mid_r = (outer_r + inner_r) / 2

            # === VẼ TRACK (NỀN XÁM) RIÊNG CHO TỪNG THANH ===
            track_pen = QPen(QColor(self.track_color), outer_r - inner_r, Qt.SolidLine, Qt.RoundCap)
            p.setPen(track_pen)
            p.setBrush(Qt.NoBrush)
            track_rect = QRectF(center.x() - mid_r, center.y() - mid_r, mid_r * 2, mid_r * 2)
            p.drawArc(track_rect.toRect(), self.start_angle * 16, self.total_angle * 16)

            # === VẼ THANH CHÍNH ===
            color = QColor(self.colors[idx % len(self.colors)])
            pen = QPen(color, outer_r - inner_r, Qt.SolidLine, Qt.RoundCap)
            p.setPen(pen)

            path = QPainterPath()
            path.arcMoveTo(track_rect, self.start_angle)
            path.arcTo(track_rect, self.start_angle, angle_span)
            p.strokePath(path, pen)

            # === VẼ NAME + VALUE Ở CUỐI VÒNG CUNG ===
            if (self.show_names or self.show_values) and value > 0:
                end_angle = self.start_angle + angle_span
                rad = radians(end_angle - 90)
                label_x = center.x() + (mid_r + 10) * cos(rad)
                label_y = center.y() + (mid_r + 10) * sin(rad)

                text = ""
                if self.show_names:
                    text += self.labels[idx]
                if self.show_values and self.show_names:
                    text += "\n"
                if self.show_values:
                    text += f"{value:,.0f}"

                p.setPen(QColor(self.theme.palette.text.secondary))
                # p.setFont(self.theme.typography.body2.font())
                fm = QFontMetrics(p.font())
                text_rect = fm.boundingRect(0, 0, 1000, 1000, Qt.AlignCenter, text)
                p.drawText(QPointF(label_x - text_rect.width() / 2, label_y + text_rect.height() / 2), text)

        # === VẼ TOTAL Ở GIỮA ===
        if self.show_total and self._series:
            total_val = sum(self._series) if self._total is None else self._total
            total_text = self.total_formatter() if callable(self.total_formatter) else f"{total_val:,.0f}"

            p.setPen(QColor(self.theme.palette.text.primary))
            # p.setFont(self.theme.typography.h5.font())
            p.drawText(draw_rect.toRect(), Qt.AlignCenter, total_text)

            # Sub label nếu có
            subtitle = self._options.get("title", {}).get("text", "")
            if subtitle:
                p.setPen(QColor(self.theme.palette.text.secondary))
                # p.setFont(self.theme.typography.body2.font())
                p.drawText(draw_rect.adjusted(0, 30, 0, -30).toRect(), Qt.AlignCenter, subtitle)

        p.end()


"""Test win32profile"""

import os
import unittest

import win32profile


class Tester(unittest.TestCase):
    def test_environment(self):
        os.environ["FOO"] = "bar=baz"
        env = win32profile.GetEnvironmentStrings()
        self.assertIn("FOO", env)
        self.assertEqual(env["FOO"], "bar=baz")
        self.assertEqual(os.environ["FOO"], "bar=baz")


if __name__ == "__main__":
    unittest.main()



from typing import Callable, Dict, Any, Optional, Union

# Định nghĩa kiểu TransformFunction, mô phỏng việc chuyển đổi giá trị dựa trên theme
TransformFunction = Callable[[Any, dict], Any]

# SimpleStyleFunction mô phỏng việc xử lý các thuộc tính style dựa trên prop
class SimpleStyleFunction:
    def __init__(self, func: Callable[[dict], dict]):
        self.func = func

    def __call__(self, props: dict) -> dict:
        return self.func(props)

# Định nghĩa SxConfigRecord, tương tự với định nghĩa trong TypeScript
class SxConfigRecord:
    def __init__(self,
                 css_property: Optional[str] = None,
                 theme_key: Optional[str] = None,
                 transform: Optional[TransformFunction] = None,
                 style: Optional[SimpleStyleFunction] = None):
        """
        :param css_property: Tên thuộc tính CSS hoặc False nếu không áp dụng
        :param theme_key: Khóa truy cập dot trong theme (nếu có)
        :param transform: Hàm chuyển đổi giá trị dựa trên theme
        :param style: Hàm xử lý các thuộc tính style dựa trên props
        """
        self.css_property = css_property
        self.theme_key = theme_key
        self.transform = transform
        self.style = style

# Định nghĩa SxConfig như một mapping giữa tên thuộc tính và SxConfigRecord
SxConfig = Dict[str, SxConfigRecord]

# Hàm xử lý mặc định cho SimpleStyleFunction
def default_style_function(props: dict) -> dict:
    # Đây là nơi bạn có thể xử lý các thuộc tính style dựa trên props
    return {key: value for key, value in props.items()}

# Tạo defaultSxConfig tương tự như defaultSxConfig trong TypeScript
defaultSxConfig: SxConfig = {
    'margin': SxConfigRecord(css_property='margin', transform=lambda value, theme: f"{value}px"),
    'padding': SxConfigRecord(css_property='padding', style=SimpleStyleFunction(default_style_function)),
    'color': SxConfigRecord(css_property='color', theme_key='palette.color', transform=lambda value, theme: theme.get('palette', {}).get('color', value))
}

# Ví dụ sử dụng defaultSxConfig
theme = {'palette': {'color': 'red'}}
sx_config_record = defaultSxConfig.get('color')

# Áp dụng transform function nếu có
if sx_config_record and sx_config_record.transform:
    transformed_value = sx_config_record.transform('blue', theme)
    # print(f"Transformed value: {transformed_value}")  # Kết quả: red (vì lấy từ theme)



"""
theme = {'palette': {'color': 'red'}}
sx_config_record = defaultSxConfig.get('color')

# Áp dụng transform function nếu có
if sx_config_record and sx_config_record.transform:
    transformed_value = sx_config_record.transform('blue', theme)
    print(f"Transformed value: {transformed_value}")  # Kết quả: red (vì lấy từ theme)


"""


from typing import Union, Callable, List, Dict, Any, Optional

# ResponsiveStyleValue có thể là một giá trị hoặc một danh sách các giá trị
ResponsiveStyleValue = Union[Any, List[Optional[Any]], Dict[str, Optional[Any]]]

# CSSPseudoSelectorProps mô phỏng các pseudo selectors của CSS như ':hover', ':focus', etc.
class CSSPseudoSelectorProps:
    def __init__(self, **kwargs: Union[Callable[[dict], 'SystemStyleObject'], 'SystemStyleObject']):
        self.selectors = kwargs

# CSSSelectorObject chứa các selectors có thể lồng vào nhau
class CSSSelectorObject:
    def __init__(self, **kwargs: Union[Callable[[dict], 'SystemStyleObject'], 'SystemStyleObject']):
        self.selectors = kwargs

# CSSSelectorObjectOrCssVariables bao gồm các CSS selectors và CSS variables
class CSSSelectorObjectOrCssVariables:
    def __init__(self, **kwargs: Union[Callable[[dict], 'SystemStyleObject'], 'SystemStyleObject', str, int]):
        self.selectors_or_variables = kwargs

# Tất cả các thuộc tính CSS từ System CSS Properties
class AllSystemCSSProperties:
    def __init__(self, **kwargs: Any):
        self.properties = kwargs

# SystemCssProperties chứa các thuộc tính CSS responsive
class SystemCssProperties:
    def __init__(self, **kwargs: Union[ResponsiveStyleValue, Callable[[dict], ResponsiveStyleValue], 'SystemStyleObject']):
        self.properties = kwargs

# SystemStyleObject định nghĩa các thuộc tính CSS tùy chỉnh
class SystemStyleObject:
    def __init__(self, **kwargs: Union[SystemCssProperties, CSSPseudoSelectorProps, CSSSelectorObjectOrCssVariables, None]):
        self.style_object = kwargs

# SxProps có thể là đối tượng hoặc một hàm trả về đối tượng SystemStyleObject
SxProps = Union[
    SystemStyleObject, 
    Callable[[dict], SystemStyleObject], 
    List[Union[bool, SystemStyleObject, Callable[[dict], SystemStyleObject]]]
]

# StyleFunctionSx mô phỏng một hàm xử lý style
class StyleFunctionSx:
    def __init__(self, filter_props: Optional[List[str]] = None):
        self.filter_props = filter_props if filter_props else []

    def __call__(self, props: dict) -> dict:
        return {}  # Ở đây bạn có thể định nghĩa logic xử lý CSS

# Hàm unstable_createStyleFunctionSx tạo style function từ mapping
def unstable_createStyleFunctionSx(styleFunctionMapping: Dict[str, StyleFunctionSx]) -> StyleFunctionSx:
    def style_function_sx(props: dict) -> dict:
        result = {}
        for key, function in styleFunctionMapping.items():
            result.update(function(props))
        return result
    
    return StyleFunctionSx()

# Một ví dụ về cách sử dụng SxProps
def example_sx_function(theme: dict) -> SystemStyleObject:
    return SystemStyleObject(color=theme.get('primary', 'black'))

# Ví dụ sử dụng StyleFunctionSx
style_function_sx = StyleFunctionSx()

# Ví dụ sử dụng với filterProps
style_function_sx.filter_props = ['color', 'margin']

# Áp dụng hàm style cho props
result = style_function_sx({'color': 'red', 'margin': '10px'})
print(result)


"""
Sử dụng:
Bạn có thể sử dụng SxProps để truyền các style responsive vào các thành phần của mình. Ví dụ:
# Định nghĩa một hàm style dựa trên theme
def custom_style_function(theme: dict) -> SystemStyleObject:
    return SystemStyleObject(backgroundColor=theme.get('background', 'white'), color=theme.get('text', 'black'))

# Áp dụng SxProps với theme
theme = {'background': 'blue', 'text': 'white'}
sx_props = custom_style_function(theme)

print(sx_props.style_object)


==> Kết quả là các thuộc tính CSS được cấu hình dựa trên theme. Bạn có thể mở rộng hoặc tích hợp cấu trúc này vào dự 
án ..site_packages.qtcompat của mình để xử lý các thuộc tính CSS động theo cách tương tự như trong MUI.
"""


"""
Mô tả các thành phần:
ResponsiveStyleValue: Mô phỏng kiểu TypeScript ResponsiveStyleValue, cho phép giá trị là một đối tượng, danh sách hoặc từ điển các giá trị CSS.

CSSPseudoSelectorProps: Mô phỏng các pseudo selectors của CSS như :hover, :focus, v.v. Đây là một lớp có thể chứa các selectors và trả về SystemStyleObject.

CSSSelectorObject & CSSSelectorObjectOrCssVariables: Mô phỏng các selectors CSS hoặc các biến CSS, cho phép xử lý các selectors hoặc biến tùy chỉnh.

SystemCssProperties: Mô phỏng các thuộc tính CSS, có thể nhận các giá trị responsive hoặc một hàm trả về giá trị dựa trên theme.

SystemStyleObject: Xử lý các thuộc tính style tùy chỉnh, kết hợp các pseudo selectors, CSS selectors, và CSS variables.

SxProps: Là kiểu dữ liệu có thể là một đối tượng SystemStyleObject, một hàm, hoặc một danh sách các giá trị hoặc hàm.

StyleFunctionSx: Đây là lớp mô phỏng chức năng xử lý các thuộc tính style dựa trên props. Nó cũng có thể chứa một danh sách filterProps để lọc các thuộc tính nhất định.

unstable_createStyleFunctionSx: Tạo ra một hàm xử lý style dựa trên một mapping giữa tên thuộc tính và hàm xử lý.
"""


from PySide6.QtCore import QObject, Property, Signal

class Locales(QObject):
    langChanged = Signal()

    def __init__(self, lang=None):
        super().__init__()
        self._lang = lang

    def getLang(self) -> dict:
        return self._lang

    def setLang(self, value):
        if self._lang != value:
            self._lang = value
            self.langChanged.emit()

    lang = Property(str, getLang, setLang, notify=langChanged)

i18n = Locales()

def changeLanguage(data: dict):
    i18n.lang = data
    

def translate(t):
    if isinstance(t, str):
        return t
    # print('tttttttttt_________________________', t())
    # print('translate_________________________', i18n.getLang())
    return i18n.getLang().get(t())


from typing import Optional, Any, Union, Callable, List, Dict

from qtmui.hooks import State, useEffect
from PySide6.QtWidgets import QFrame, QHBoxLayout

from ..avatar import Avatar
from qtmui.material.styles import useTheme
from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
from ..utils.validate_params import _validate_param

class AvatarGroup(QFrame):
    """
    A component that displays a group of avatars with customizable spacing and surplus rendering.

    The `AvatarGroup` component stacks avatars horizontally, limiting the number displayed based on
    the `max` prop and rendering surplus avatars using a custom renderer or a default `+x` avatar.
    It supports all props of the Material-UI `AvatarGroup` component, including native component
    props via `**kwargs`.

    Parameters
    ----------
    children : State or List[Avatar], optional
        The avatars to stack. Must be a list of `Avatar` components. Default is None.
        Can be a `State` object for dynamic updates.
    classes : State or dict, optional
        Override or extend the styles applied to the component. Default is None.
        Can be a `State` object for dynamic updates.
    component : State or Any, optional
        Component used for the root node (e.g., HTML element or custom component).
        Default is None. Can be a `State` object for dynamic updates.
    componentsProps : State or dict, optional
        Extra props for slot components (e.g., additionalAvatar). Deprecated: Use `slotProps` instead.
        Default is None. Can be a `State` object for dynamic updates.
    max : State or int, optional
        Maximum number of avatars to show before displaying a surplus indicator. Default is 5.
        Can be a `State` object for dynamic updates.
    renderSurplus : State or Callable, optional
        Custom renderer for surplus avatars. Signature: function(surplus: int) => Any.
        Default is None (uses default `+x` Avatar). Can be a `State` object for dynamic updates.
    slotProps : State or dict, optional
        Props for each slot (e.g., additionalAvatar, surplus). Default is None.
        Can be a `State` object for dynamic updates.
    slots : State or dict, optional
        Components for each slot (e.g., surplus). Default is None.
        Can be a `State` object for dynamic updates.
    spacing : State, str, or int, optional
        Spacing between avatars. Valid values: 'small' (4px), 'medium' (8px), or a number.
        Default is 'medium'. Can be a `State` object for dynamic updates.
    sx : State, dict, Callable, str, or None, optional
        System prop for CSS overrides and additional styles. Default is None.
        Can be a `State` object for dynamic updates.
    total : State or int, optional
        Total number of avatars, used to calculate surplus. Defaults to len(children) if not set.
        Can be a `State` object for dynamic updates.
    variant : State or str, optional
        Shape of the avatars. Valid values: 'circular', 'rounded', 'square'. Default is 'circular'.
        Can be a `State` object for dynamic updates.
    **kwargs
        Additional keyword arguments passed to the parent `QFrame` class, supporting
        native component props (e.g., style, className).

    Attributes
    ----------
    VALID_VARIANTS : list[str]
        Valid values for the `variant` parameter: ["circular", "rounded", "square"].
    VALID_SPACING : list[str]
        Valid string values for the `spacing` parameter: ["small", "medium"].

    Notes
    -----
    - Props of the native component are supported via `**kwargs` (e.g., `style`, `className`).
    - The `componentsProps` prop is deprecated; use `slotProps` instead.
    - If `renderSurplus` is not provided, surplus avatars are rendered as an `Avatar` with text `+x`.

    Demos:
    - AvatarGroup: https://qtmui.com/material-ui/qtmui-avatar-group/

    API Reference:
    - AvatarGroup API: https://qtmui.com/material-ui/api/avatar-group/
    """

    VALID_VARIANTS = ["circular", "rounded", "square"]
    VALID_SPACING = ["small", "medium"]

    def __init__(
        self,
        children: Optional[Union[State, List[Avatar]]] = None,
        classes: Optional[Union[State, Dict]] = None,
        component: Optional[Union[State, Any]] = None,
        componentsProps: Optional[Union[State, Dict]] = None,
        max: Union[State, int] = 5,
        renderSurplus: Optional[Union[State, Callable[[int], Any]]] = None,
        slotProps: Optional[Union[State, Dict[str, Union[Callable, Dict]]]] = None,
        slots: Optional[Union[State, Dict[str, Any]]] = None,
        spacing: Union[State, str, int] = "medium",
        sx: Optional[Union[State, Dict, Callable, str]] = None,
        total: Optional[Union[State, int]] = None,
        variant: Union[State, str] = "circular",
        **kwargs
    ):
        super().__init__(**kwargs)

        # Set properties with validation
        self._set_children(children)
        self._set_classes(classes)
        self._set_component(component)
        self._set_componentsProps(componentsProps)
        self._set_max(max)
        self._set_renderSurplus(renderSurplus)
        self._set_slotProps(slotProps)
        self._set_slots(slots)
        self._set_spacing(spacing)
        self._set_sx(sx)
        self._set_total(total)
        self._set_variant(variant)

        self._init_ui()
        # self._set_stylesheet()

        self.theme = useTheme()
        # useEffect(
        #     self._set_stylesheet,
        #     [self.theme.state]
        # )
        # self.destroyed.connect(self._on_destroyed)

    # Setter and Getter methods for all parameters
    # @_validate_param(file_path="qtmui.material.avatar_group", param_name="children", supported_signatures=Union[State, List[Avatar], type(None)])
    def _set_children(self, value):
        self._children = value

    def _get_children(self):
        return self._children.value if isinstance(self._children, State) else self._children

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
    def _set_classes(self, value):
        self._classes = value

    def _get_classes(self):
        return self._classes.value if isinstance(self._classes, State) else self._classes

    # @_validate_param(file_path="qtmui.material.avatar_group", param_name="component", supported_signatures=Union[State, Any, type(None)])
    def _set_component(self, value):
        self._component = value

    def _get_component(self):
        return self._component.value if isinstance(self._component, State) else self._component

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="componentsProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_componentsProps(self, value):
        self._componentsProps = value

    def _get_componentsProps(self):
        return self._componentsProps.value if isinstance(self._componentsProps, State) else self._componentsProps

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="max", supported_signatures=Union[State, int])
    def _set_max(self, value):
        self._max = value

    def _get_max(self):
        return self._max.value if isinstance(self._max, State) else self._max

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="renderSurplus", supported_signatures=Union[State, Callable, type(None)])
    def _set_renderSurplus(self, value):
        self._renderSurplus = value

    def _get_renderSurplus(self):
        return self._renderSurplus.value if isinstance(self._renderSurplus, State) else self._renderSurplus

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
    def _set_slotProps(self, value):
        self._slotProps = value or {}

    def _get_slotProps(self):
        return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
    def _set_slots(self, value):
        self._slots = value or {}

    def _get_slots(self):
        return self._slots.value if isinstance(self._slots, State) else self._slots

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="spacing", supported_signatures=Union[State, str, int], valid_values=VALID_SPACING)
    def _set_spacing(self, value):
        self._spacing = value

    def _get_spacing(self):
        return self._spacing.value if isinstance(self._spacing, State) else self._spacing

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="sx", supported_signatures=Union[State, Dict, Callable, str, type(None)])
    def _set_sx(self, value):
        self._sx = value

    def _get_sx(self):
        return self._sx.value if isinstance(self._sx, State) else self._sx

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="total", supported_signatures=Union[State, int, type(None)])
    def _set_total(self, value):
        if value is not None:
            self._total = value
        else:
            children = self._get_children()
            self._total = len(children) if children else 0

    def _get_total(self):
        return self._total.value if isinstance(self._total, State) else self._total

    @_validate_param(file_path="qtmui.material.avatar_group", param_name="variant", supported_signatures=Union[State, str], valid_values=VALID_VARIANTS)
    def _set_variant(self, value):
        self._variant = value

    def _init_ui(self):
        self.setLayout(QHBoxLayout())
        avatar_count = 0
        avatar_w = 0
        avatar_size = 0
        total_w = 0

        self.layout().addWidget(Avatar(text=f"+{len(self._children) - 3}", customText=True, size=self._children[0]._size, color="primary", borderWidth=1))
        for avatar in self._children:
            if isinstance(avatar, Avatar):
                total_w  += avatar.sizeHint().width()
                if avatar_w == 0:
                    avatar_w = total_w
                    avatar_size = avatar._size
                self.layout().addWidget(avatar)
                avatar_count += 1
                if avatar_count == 3:
                    break

        # self.layout().insertWidget(-1, Avatar(text=f"+{len(self._children) - avatar_count}", customText=True, size=avatar_size, color="primary"))
        widgets = self.findChildren(Avatar)
        # widgets.reverse()
        for index, widget in enumerate(widgets, 0):
            # if index < 3:
                self.layout().insertWidget(0, widget)
            # else:
            #     self.layout().insertWidget(-1, widget)

        self.layout().setContentsMargins(0,0,avatar_w,0)

        if len(self._children) > 1:
            self.setFixedWidth(total_w)

    # Getter và Setter cho các thuộc tính

    @property
    def children(self) -> Optional[Any]:
        """Các avatar cần được xếp chồng lên nhau."""
        return self._children

    @children.setter
    def children(self, value: Optional[Any]):
        self._children = value

    @property
    def classes(self) -> Optional[Dict]:
        """Ghi đè hoặc mở rộng các style áp dụng cho component."""
        return self._classes

    @classes.setter
    def classes(self, value: Optional[Dict]):
        self._classes = value

    @property
    def component(self) -> Optional[Any]:
        """Component được sử dụng cho node gốc, có thể là HTML element hoặc component."""
        return self._component

    @component.setter
    def component(self, value: Optional[Any]):
        self._component = value

    @property
    def componentsProps(self) -> Optional[Dict[str, Any]]:
        """Props bổ sung cho các slot component (đã deprecated)."""
        return self._componentsProps

    @componentsProps.setter
    def componentsProps(self, value: Optional[Dict[str, Any]]):
        self._componentsProps = value

    @property
    def max(self) -> Optional[int]:
        """Số lượng avatar tối đa trước khi hiển thị dấu +x."""
        return self._max

    @max.setter
    def max(self, value: Optional[int]):
        self._max = value

    @property
    def renderSurplus(self) -> Optional[Callable[[int], Any]]:
        """Hàm custom render cho các avatar dư thừa."""
        return self._renderSurplus

    @renderSurplus.setter
    def renderSurplus(self, value: Optional[Callable[[int], Any]]):
        self._renderSurplus = value

    @property
    def slotProps(self) -> Optional[Dict[str, Union[Callable, Dict]]]:
        """Thuộc tính sử dụng cho mỗi slot bên trong AvatarGroup."""
        return self._slotProps

    @slotProps.setter
    def slotProps(self, value: Optional[Dict[str, Union[Callable, Dict]]]):
        self._slotProps = value

    @property
    def slots(self) -> Optional[Dict[str, Any]]:
        """Component sử dụng cho mỗi slot bên trong AvatarGroup."""
        return self._slots

    @slots.setter
    def slots(self, value: Optional[Dict[str, Any]]):
        self._slots = value

    @property
    def spacing(self) -> Union[str, int]:
        """Khoảng cách giữa các avatar ('medium', 'small', hoặc số)."""
        return self._spacing

    @spacing.setter
    def spacing(self, value: Union[str, int]):
        self._spacing = value

    @property
    def sx(self) -> Optional[Union[List[Union[Callable, Dict, bool]], Callable, Dict, bool]]:
        """Prop hệ thống cho phép định nghĩa CSS overrides hoặc thêm CSS styles."""
        return self._sx

    @sx.setter
    def sx(self, value: Optional[Union[List[Union[Callable, Dict, bool]], Callable, Dict, bool]]):
        self._sx = value

    @property
    def total(self) -> Optional[int]:
        """Tổng số avatar, được sử dụng để tính số lượng avatar dư thừa."""
        return self._total

    @total.setter
    def total(self, value: Optional[int]):
        self._total = value

    @property
    def variant(self) -> str:
        """Kiểu của avatar ('circular', 'rounded', 'square', hoặc string)."""
        return self._variant

    @variant.setter
    def variant(self, value: str):
        self._variant = value



from .list import List
from .list_item import ListItem
from .list_item_avatar import ListItemAvatar
from .list_item_button import ListItemButton
from .list_item_checkbox import ListItemCheckbox
from .list_item_secondary_action import ListItemSecondaryAction
from .list_item_text import ListItemText
from .list_subheader import ListSubheader
from .list_item_icon import ListItemIcon


from typing import Callable, Optional, Union
import uuid

from PySide6.QtWidgets import QHBoxLayout, QFrame, QPushButton

from qtmui.hooks import State

from ...material.styles import useTheme

from ..widget_base import PyWidgetBase

class Option(QFrame, PyWidgetBase):
    def __init__(self,
                 key: str = None,
                 value: object = None,
                 enabled: bool = True,
                label: Optional[Union[str, State, Callable]] = None,
                 
                 selected: bool = None,
                 children: object = None,
                 *args, **kwargs
                 ):
        super().__init__(*args, **kwargs)

        self._enabled = enabled
        self._children = children
        self._key = key
        self._label = label
        self._value = value
        self._selected = value

        self.__init_ui()

        PyWidgetBase._installTooltipFilter(self)

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()


    def __init_ui(self):

        self.setLayout(QHBoxLayout())
        self.layout().setContentsMargins(0,0,0,0)

        if self._children is not None:
            if isinstance(self._children, list):
                for item in self._children:
                    self.layout().addWidget(item)
            else:
                self.layout().addWidget(self._children)
        else:
            self.button = QPushButton(text=self._label if self._label else "")

            self.layout().addWidget(self.button)
            self.clicked = self.button.clicked


    def _set_stylesheet(self):
        theme = useTheme()

        self.button.setStyleSheet(f"""
            QPushButton {{
                padding: 10px;
                border-radius: {theme.shape.borderRadius}px;
                background-color: transparent;
                color: {theme.palette.text.secondary};
                text-align: left;
                font-weight: {theme.typography.button.fontWeight};
                line-height: {theme.typography.button.lineHeight};
                font-size: {theme.typography.button.fontSize};
            }}
            QPushButton:hover {{
                padding: 10px;
                border-radius: {theme.shape.borderRadius}px;
                background-color: {theme.palette.action.selected};
                color: {theme.palette.text.secondary};
            }}
        """)

    def set_selected(self, state):
        self._selected = state
        self._set_selected(state)

    def _set_selected(self, selected):
        theme = useTheme()

        if selected:
            self.button.setStyleSheet(f"""
                QPushButton {{
                    padding: 10px;
                    border-radius: {theme.shape.borderRadius}px;
                    background-color: {theme.palette.action.selected};
                    color: {theme.palette.text.secondary};
                    text-align: left;
                    font-weight: {theme.typography.button.fontWeight};
                    line-height: {theme.typography.button.lineHeight};
                    font-size: {theme.typography.button.fontSize};
                }}
                QPushButton:hover {{
                    padding: 10px;
                    border-radius: {theme.shape.borderRadius}px;
                    background-color: {theme.palette.action.selected};
                    color: {theme.palette.text.secondary};
                }}
            """)
        else:
            self.button.setStyleSheet(f"""
                QPushButton {{
                    padding: 10px;
                    border-radius: {theme.shape.borderRadius}px;
                    background-color: transparent;
                    color: {theme.palette.text.secondary};
                    text-align: left;
                    font-weight: {theme.typography.button.fontWeight};
                    line-height: {theme.typography.button.lineHeight};
                    font-size: {theme.typography.button.fontSize};
                }}
                QPushButton:hover {{
                    padding: 10px;
                    border-radius: {theme.shape.borderRadius}px;
                    background-color: {theme.palette.action.selected};
                    color: {theme.palette.text.secondary};
                }}
            """)

    def set_menu_item_visible(self, state):
        self.button.set_menu_item_visible(state)
        # if state == True:
        #     self.parent().setMaximumHeight(self.parent().height() + self._height)
        # else:
        #     self.parent().setMaximumHeight(self.parent().height() - self._height)




from typing import Callable, Dict
import asyncio
from PySide6.QtCore import QAbstractTableModel, QObject, Signal, Property, QModelIndex, QTimer, QThreadPool, QRunnable
from PySide6.QtGui import Qt
from PySide6.QtWidgets import QScrollBar, QStyledItemDelegate, QPushButton, QTableView, QHBoxLayout, QMainWindow, QFrame, QHeaderView, QApplication

from qtmui.hooks import State
from ..skeleton import Skeleton
from ..box import Box

INDEXS = []
SCROLL_POS = None


class StyledItem(QStyledItemDelegate):
    def __init__(self, parent=None, model=None):
        super(StyledItem, self).__init__(parent)
        self._model = model

    def paint(self, painter, option, index):
        super(StyledItem, self).paint(painter, option, index)
        if not self.parent().indexWidget(index) and not index.parent().isValid():
            pass  # Logic đã được chuyển sang _set_index_widget


class GridViewModel(QAbstractTableModel):
    def __init__(self, parent, data):
        super(GridViewModel, self).__init__(parent)
        self._data = data

    def rowCount(self, n=None):
        return len(self._data)

    def columnCount(self, n=None):
        return len(self._data[0])

    def data(self, index, role=Qt.ItemDataRole.DisplayRole):
        if index.isValid():
            if role == Qt.ItemDataRole.DisplayRole or role == Qt.ItemDataRole.EditRole:
                try:
                    widget_info = self._data[index.row()][index.column()]
                    return widget_info
                except:
                    return None


# Worker để chạy setIndexWidget trong thread riêng
class WidgetSetter(QRunnable):
    def __init__(self):
        super().__init__()

    async def run(self, grid_view, index, widget):
        if not grid_view.indexWidget(index):
            grid_view.setIndexWidget(index, widget)


class GridView(QTableView):
    setIndexWidgetSignal = Signal(object)
    removeIndexWidgetSignal = Signal(QModelIndex)

    def __init__(
            self,
            model: State,
            columnCount=None,
            rowHeight=50,
            skeleton: Callable = None,
            sm=1,
            md=2,
            lg=3,
            xl=4
    ):
        super(GridView, self).__init__()
        self._rowHeight = rowHeight
        self._columnCount = columnCount
        self._skeleton = skeleton
        self.sm = sm
        self.md = md
        self.lg = lg
        self.xl = xl

        self.model_data = []
        self._last_visible_indexes = []

        if self._columnCount:
            self.columnsToShow = self._columnCount
        else:
            self.columnsToShow = sm

        self._model: State = model
        self._thread_pool = QThreadPool.globalInstance()  # Sử dụng thread pool toàn cục
        self.worker = WidgetSetter()

        self.verticalHeader().hide()
        self.horizontalHeader().hide()
        self.setDragEnabled(False)
        self.setAcceptDrops(False)

        self._model.valueChanged.connect(self._set_model_data)
        self._set_stylesheet()

        self.setIndexWidgetSignal.connect(self._set_index_widget)
        self.removeIndexWidgetSignal.connect(self._remove_index_widget)

        self._setup_model()

    def _setup_model(self):
        data = self._split_data_by_breakpoint()
        self._gridViewModel = GridViewModel(self, data=data)
        self.setModel(self._gridViewModel)

        self._set_all_column_stretch_mode()
        self._thread_setup_table_for_visible_indexes()
        self.viewport().update()

    def _thread_setup_table_for_visible_indexes(self):
        global SCROLL_POS
        scroll_position = self.verticalScrollBar().value()
        if SCROLL_POS == scroll_position:
            return
        SCROLL_POS = scroll_position
        self._setup_table_for_visible_indexes()

    def _setup_table_for_visible_indexes(self):
        model = self.model()
        if not model:
            return

        viewport_rect = self.viewport().rect()
        first_row = self.rowAt(viewport_rect.top())
        last_row = self.rowAt(viewport_rect.bottom())
        first_col = self.columnAt(viewport_rect.left())
        last_col = self.columnAt(viewport_rect.right())

        if first_row == -1: first_row = 0
        if last_row == -1: last_row = model.rowCount() - 1
        if first_col == -1: first_col = 0
        if last_col == -1: last_col = model.columnCount() - 1

        visible_indexes = [
            model.index(row, col)
            for row in range(first_row, last_row + 1)
            for col in range(first_col, last_col + 1)
        ]

        for index in self._last_visible_indexes:
            if index not in visible_indexes:
                self.removeIndexWidgetSignal.emit(index)

        self.setIndexWidgetSignal.emit(visible_indexes)
        self._last_visible_indexes = visible_indexes

    def _remove_index_widget(self, index):
        if self.indexWidget(index):
            self.indexWidget(index).deleteLater()
            self.setIndexWidget(index, None)

    def _set_index_widget(self, indexes):
        index_rows = []
        new_visible_indexes = []
        for index in indexes:
            try:
                if not self.indexWidget(index) and not index.parent().isValid():
                    index_row = index.row()
                    index_col = index.column()
                    if index_row not in index_rows:
                        index_rows.append(index_row)

                    new_visible_indexes.append(f"{index_row}{index_col}")

                    widget_info = self.model_data[index_row][index_col]
                    widget = None
                    if isinstance(widget_info, Callable):
                        widget = widget_info()
                    elif isinstance(widget_info, Dict):
                        widget = widget_info["component"](**widget_info["renderProps"])

                    if widget:
                        # self.setIndexWidget(index, widget) # Không làm tăng RAM
                        self._thread_pool.start(QTimer.singleShot(0, lambda index=index, widget=widget: asyncio.ensure_future(self.worker.run(self, index, widget))))  # Chạy setIndexWidget trong thread riêng

                    for row in index_rows:
                        self.setRowHeight(row, self._rowHeight)
            except Exception as e:
                print(f"Error in _set_index_widget: {e}")


    def _set_stylesheet(self):
        self.setStyleSheet("""
            QTableView {
                padding: 0px;
                border: none;
                gridline-color: transparent;
                color: transparent;
            }
            QTableView::item {
                padding-left: 0px;
                padding-right: 0px;
            }
            QTableView::item:selected {
                background-color: transparent;
                color: transparent;
            }
            QTableView::section {
                background-color: transparent;
                text-align: left;
            }
            QTableView::horizontalHeader {
                background-color: transparent;
            }
            QTableView::section:horizontal {
                background-color: transparent;
                padding: 0px;
                border: 1px solid transparent !important;
                border-bottom: 1px solid transparent !important;
            }
            QTableView::section:vertical {
                border: 1px solid transparent;
            }
            QTableView .QScrollBar:horizontal {
                border: none;
                background: transparent;
                border-radius: 0px;
            }
        """)

    def _set_model_data(self, _data):
        data = self._split_data_by_breakpoint()
        self._gridViewModel = GridViewModel(self, data=data)
        self.setModel(self._gridViewModel)
        self._set_all_column_stretch_mode()
        self._setup_table_for_visible_indexes()
        self.viewport().update()

    def _split_data_by_breakpoint(self):
        data = self._model.value
        new_data = []
        temp_row = []

        for i, item in enumerate(data):
            temp_row.append(item)
            if len(temp_row) == self.columnsToShow:
                new_data.append(temp_row)
                temp_row = []

        if temp_row:
            new_data.append(temp_row)

        for i in range(self.columnsToShow):
            self.setItemDelegateForColumn(i, StyledItem(self, new_data))

        self.model_data = new_data
        return new_data

    def _set_all_column_stretch_mode(self):
        for i in range(self.columnsToShow):
            self.horizontalHeader().setSectionResizeMode(i, QHeaderView.Stretch)

    def resizeEvent(self, event):
        width = self.width()
        if width < 600:
            self.columnsToShow = self.sm
        elif width < 960:
            self.columnsToShow = self.md
        elif width < 1280:
            self.columnsToShow = self.lg
        else:
            self.columnsToShow = self.xl

        if self._columnCount:
            self.columnsToShow = self._columnCount

        self._set_model_data(self._model)
        super().resizeEvent(event)

    def wheelEvent(self, event):
        super().wheelEvent(event)
        self._thread_setup_table_for_visible_indexes()




# src/qtmui/hooks/_effect_runner.py
from typing import Callable, List, Optional
from PySide6.QtCore import QObject
from .use_state import State


class _EffectRunner(QObject):
    def __init__(self, callback: Callable, deps: List[State]):
        super().__init__()
        self.callback = callback
        self.deps = deps or []
        self.cleanup: Optional[Callable] = None
        self.connections = []
        self._setup()

    def _setup(self):
        for dep in self.deps:
            if isinstance(dep, State):
                conn = dep.valueChanged.connect(self.run)
                self.connections.append((dep, conn))
        # self.run()  # lần đầu

    def run(self):
        # Cleanup cũ
        if self.cleanup and callable(self.cleanup):
            try:
                self.cleanup()
            except Exception as e:
                print(f"[useEffect] Cleanup error: {e}")
            self.cleanup = None

        # Chạy mới
        try:
            result = self.callback()
            if callable(result):
                self.cleanup = result
        except Exception as e:
            print(f"[useEffect] Error: {e}")

    def destroy(self):
        if self.cleanup and callable(self.cleanup):
            try:
                self.cleanup()
            except Exception:
                pass

        for dep, conn in self.connections:
            try:
                dep.valueChanged.disconnect(conn)
            except RuntimeWarning as e:
                pass
            
        try:
            self.deleteLater()
        except Exception as e:
            pass


from __future__ import annotations

from typing import Callable, Optional, Union, List, Dict
import math

from PySide6.QtCore import Qt, QPointF
from PySide6.QtWidgets import QWidget, QVBoxLayout
from PySide6.QtCharts import (
    QChart,
    QChartView,
    QLineSeries,
    QValueAxis,
    QCategoryAxis,
    QPolarChart,
    QAreaSeries,
    QBarSet,
    QBarSeries,
    QBarCategoryAxis
)
from PySide6.QtGui import QPainter, QColor, QBrush, QPen, QFont

from qtmui.hooks import State
from qtmui.material.styles import useTheme


class ChartPolarArea(QWidget):
    def __init__(
        self,
        dir: str = "ltr",
        series: List[int] = [],  # Thay đổi kiểu dữ liệu của series
        width: Optional[Union[str, int]] = None,
        height: Optional[Union[str, int]] = None,
        options: Optional[Dict] = None,
        key: str = None,
        title: Optional[Union[State, str, Callable]] = None,
        *args,
        **kwargs,
    ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        self.series = series
        self.options = options or {}
        self.title = title
        self._height = height
        self._width = width

        self._init_polar_area_chart()

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _init_polar_area_chart(self):
        self.theme = useTheme()
        self.chart = QPolarChart()

        # Lấy labels từ options, nếu không có thì dùng list rỗng
        self.labels = self.options.get("labels", [])

        if self._width:
            self.setFixedWidth(self._width)
        if self._height:
            self.setFixedHeight(self._height)

        # Tạo các trục tọa độ cực
        self.angular_axis = self._create_angular_axis()
        self.radial_axis = self._create_radial_axis()

        # Thêm các trục vào chart
        self.chart.addAxis(self.angular_axis, QPolarChart.PolarOrientationAngular)
        self.chart.addAxis(self.radial_axis, QPolarChart.PolarOrientationRadial)

        # Thêm các series dữ liệu
        self._add_data_series()

        # Cấu hình chart
        if self.title:
            self.chart.setTitle(self.title)

        self.chart.legend().setAlignment(Qt.AlignBottom)
        self.chart.legend().show()

        # Tạo ChartView
        self.chart_view = QChartView(self.chart)
        self.chart_view.setRenderHint(QPainter.Antialiasing)
        self.layout().addWidget(self.chart_view)

    def _create_angular_axis(self):
        axis = QCategoryAxis()
        angle_step = 360 / len(self.labels) if self.labels else 0

        # Tạo nhãn theo thứ tự ban đầu:
        for i, label in enumerate(self.labels):
            angle = i * angle_step  # Giá trị góc tính theo 0, 60, 120, ... nếu có 6 label
            axis.append(label, angle)

        axis.setRange(0, 360)
        axis.setLabelsFont(QFont("Arial", 10))
        return axis


    def _create_radial_axis(self):
        axis = QValueAxis()
        axis.setTickCount(5)
        axis.setLabelFormat("%d")
        max_value = max(self.series) if self.series else 100
        axis.setRange(0, max_value * 1.2)  # Điều chỉnh range theo giá trị max
        return axis

    def _add_data_series(self):
        colors = self.options.get("colors", [self.theme.palette.primary.main])
        fill_opacity = self.options.get("fill", {}).get("opacity", 0.8)

        max_value = max(self.series) if self.series else 100

        # Tạo QAreaSeries cho biểu đồ vùng cực
        series = QAreaSeries()

        # Tạo QLineSeries cho đường viền
        line_series = QLineSeries()

        angle_step = 360 / len(self.series)
        for i, value in enumerate(self.series):
            # Tính góc: 0, 60, 120, ...
            angle = i * angle_step
            radius = (value)  # Sử dụng trực tiếp giá trị

            line_series.append(QPointF(angle, radius))


        series.setUpperSeries(line_series)

        # Tạo màu cho khu vực
        for i, value in enumerate(self.series):
            color = QColor(colors[i % len(colors)])
            brush = QBrush(color)
            # brush.setOpacity(fill_opacity)
            series.setBrush(brush)

            # Không vẽ đường viền, chỉ fill màu
            pen = QPen(color)
            pen.setWidth(0)
            series.setPen(pen)

        self.chart.addSeries(series)
        series.attachAxis(self.angular_axis)
        series.attachAxis(self.radial_axis)


    def _set_stylesheet(self):
        theme = useTheme()
        # Cấu hình màu sắc
        self.chart.setBackgroundBrush(QBrush(QColor(theme.palette.background.paper)))
        self.chart.setTitleBrush(QBrush(QColor(theme.palette.text.primary)))

        # Cập nhật màu cho các trục
        text_color = QColor(theme.palette.text.secondary)
        for axis in self.chart.axes():
            axis.setLabelsBrush(QBrush(text_color))
            axis.setLinePenColor(text_color)
            axis.setGridLineColor(QColor(theme.palette.divider))

        # Cập nhật legend
        self.chart.legend().setLabelColor(text_color)



# Copyright (C) 2022 The Pyact Company Ltd.
# SPDX-License-Identifier: LicenseRef-Pyact-Commercial OR BSD-3-Clause
from __future__ import annotations

"""..site_packages.qtcompat port of the areachart example from qtmui v1.0"""

from typing import Optional, Union
import sys
from PySide6.QtCore import QPointF, Qt
from PySide6.QtWidgets import QMainWindow, QApplication, QWidget, QVBoxLayout
from PySide6.QtCharts import (
    QChart, 
    QChartView, 
    QLineSeries, 
    QBarCategoryAxis, 
    QChart,
    QChartView, 
)
from PySide6.QtGui import QGradient, QPen, QLinearGradient, QPainter, QColor, QBrush
from ..system.color_manipulator import alpha
from qtmui.material.styles import useTheme


class ChartLine(QWidget):
    def __init__(
        self,
        dir: str = "ltr",  # "line" | "area" | "bar" | "pie" | "donut" | "radialBar" | "scatter" | "bubble" | "heatmap" | "candlestick" | "boxPlot" | "radar" | "polarArea" | "rangeBar" | "rangeArea" | "treemap"
        type: str = "line",  # "line" | "area" | "bar" | "pie" | "donut" | "radialBar" | "scatter" | "bubble" | "heatmap" | "candlestick" | "boxPlot" | "radar" | "polarArea" | "rangeBar" | "rangeArea" | "treemap"
        series: object = None, 
        width: Optional[Union[str, int]] = None, 
        height: Optional[Union[str, int]] = None, 
        options: object = None,
        key: str = None,
        *args,
        **kwargs
    ):
        super().__init__(*args, **kwargs)
        self.setLayout(QVBoxLayout())
        self.layout().setContentsMargins(0, 0, 0, 0)

        self._type = type
        self._series = series
        self._width = width
        self._height = height
        self._options = options

        self._init_line_chart()

        theme = useTheme()
        useEffect(
            self._set_stylesheet,
            [theme.state]
        )
        self._set_stylesheet()

    def _init_line_chart(self):
        self.theme = useTheme()

        if self._height:
            self.setFixedHeight(self._height)
        if self._width:
            self.setFixedWidth(self._width)

        self._line_series = []

        # Tạo một QLineSeries cho mỗi phần tử trong series
        for i, item in enumerate(self._series):
            series = QLineSeries()
            series.setName(item.get("name") or "")  # Đặt tên cho legend nếu có
            data = item.get("data", [])
            # Kiểm tra nếu mỗi điểm dữ liệu là dict chứa "x" và "y"
            for point in data:
                if isinstance(point, dict):
                    try:
                        x = float(point.get("x", 0))
                        y = float(point.get("y", 0))
                    except (ValueError, TypeError):
                        x, y = 0.0, 0.0
                else:
                    # Nếu không phải dict, hãy sử dụng index và giá trị đó (nếu phù hợp)
                    # Tuy nhiên, với đầu vào theo yêu cầu, chúng ta mong đợi dict.
                    x, y = 0.0, float(point)
                series.append(QPointF(x, y))
            self._line_series.append(series)

        # Tạo chart và thêm các series vào chart
        self.chart = QChart()
        for series in self._line_series:
            self.chart.addSeries(series)

        # Hiển thị legend và căn chỉnh vị trí của legend
        self.chart.legend().setVisible(True)
        self.chart.legend().setAlignment(Qt.AlignTop)

        # Tạo các trục mặc định cho chart
        self.chart.createDefaultAxes()

        # Nếu có tùy chọn x-axis categories, đặt lại trục x
        if self._options.get("xaxis") and "categories" in self._options["xaxis"]:
            axis_x = QBarCategoryAxis()
            axis_x.append(self._options["xaxis"]["categories"])
            self.chart.setAxisX(axis_x, self._line_series[0])
        elif self._options.get("categories"):
            axis_x = QBarCategoryAxis()
            axis_x.append(self._options["categories"])
            self.chart.setAxisX(axis_x, self._line_series[0])

        # Thiết lập ChartView
        self._chart_view = QChartView(self.chart)
        self._chart_view.setRenderHint(QPainter.RenderHint.Antialiasing)
        self.layout().addWidget(self._chart_view)

    def _set_stylesheet(self):
        theme = useTheme()

        text_color = QColor(self.theme.palette.text.secondary)
        # 1. Màu cho tiêu đề
        self.chart.setTitleBrush(text_color)

        # 2. Màu cho nhãn trục
        for axis in self.chart.axes():
            axis.setLabelsBrush(text_color)

        # 3. Màu cho nhãn trong legend (nếu bật)
        legend = self.chart.legend()
        legend.setLabelBrush(text_color)

        # Set the chart's background
        gradient = QLinearGradient(QPointF(0, 0), QPointF(0, 1))
        gradient.setColorAt(0.0, QColor(theme.palette.background.paper))  # Top color
        gradient.setColorAt(1.0, QColor(theme.palette.background.main))  # Bottom color
        gradient.setCoordinateMode(QGradient.ObjectBoundingMode)
        self.chart.setBackgroundBrush(gradient)
        self.chart.setBackgroundRoundness(10)  # Optional: Rounded edges for the background


# import threading
# import time
# from typing import Callable, Optional, Union, List, Dict, Any
# from PySide6.QtWidgets import QWidget, QVBoxLayout, QLineEdit
# from PySide6.QtCore import Signal, QTimer
# from qtmui.material.styles import useTheme
# from qtmui.material.styles.create_theme.theme_reducer import ThemeState
# from ..system.color_manipulator import alpha
# from qtmui.hooks import State
# from ..textfield import TextField
# from ..utils.validate_params import _validate_param

# class Select(QWidget):
#     """
#     A select component, styled like Material-UI Select.

#     The `Select` component allows users to choose one or more options from a dropdown menu. It integrates
#     with the `qtmui` framework, retaining existing parameters, adding new parameters, and aligning with
#     MUI Select props. Inherits from OutlinedInput props.

#     Parameters
#     ----------
#     options : State or List, optional
#         The option elements to populate the select with. Default is [].
#         Can be a `State` object for dynamic updates.
#     renderInput : State or Callable, optional
#         Function to render the input component. Default is None.
#         Can be a `State` object for dynamic updates.
#     autoComplete : State or bool, optional
#         If True, enables autocomplete. Default is False.
#         Can be a `State` object for dynamic updates.
#     autoHighlight : State or bool, optional
#         If True, automatically highlights the first option. Default is False.
#         Can be a `State` object for dynamic updates.
#     autoSelect : State or bool, optional
#         If True, auto-selects value when input loses focus. Default is False.
#         Can be a `State` object for dynamic updates.
#     blurOnSelect : State or bool, optional
#         If True, blurs input after selection. Default is False.
#         Can be a `State` object for dynamic updates.
#     ChipProps : State or Dict, optional
#         Properties for chip elements. Default is None.
#         Can be a `State` object for dynamic updates.
#     classes : State or Dict, optional
#         Override or extend the styles applied to the component. Default is None.
#         Can be a `State` object for dynamic updates.
#     clearIcon : State or QWidget, optional
#         Icon for clear action. Default is None.
#         Can be a `State` object for dynamic updates.
#     clearOnBlur : State or bool, optional
#         If True, clears input on blur. Default is False.
#         Can be a `State` object for dynamic updates.
#     clearOnEscape : State or bool, optional
#         If True, clears input on escape key. Default is False.
#         Can be a `State` object for dynamic updates.
#     clearText : State or str, optional
#         Text for clear button. Default is 'Clear'.
#         Can be a `State` object for dynamic updates.
#     closeText : State or str, optional
#         Text for close button. Default is 'Close'.
#         Can be a `State` object for dynamic updates.
#     children : State or List[QWidget], optional
#         MenuItem elements for the select. Default is None.
#         Can be a `State` object for dynamic updates.
#     componentsProps : State or Dict, optional
#         Props for sub-components. Default is None.
#         Can be a `State` object for dynamic updates.
#     defaultValue : State or Any, optional
#         The default value when not controlled. Default is None.
#         Can be a `State` object for dynamic updates.
#     disableClearable : State or bool, optional
#         If True, disables clear action. Default is False.
#         Can be a `State` object for dynamic updates.
#     disableCloseOnSelect : State or bool, optional
#         If True, keeps popup open after selection. Default is False.
#         Can be a `State` object for dynamic updates.
#     disabled : State or bool, optional
#         If True, disables the component. Default is False.
#         Can be a `State` object for dynamic updates.
#     disabledItemsFocusable : State or bool, optional
#         If True, allows focusing disabled items. Default is False.
#         Can be a `State` object for dynamic updates.
#     disableListWrap : State or bool, optional
#         If True, disables list wrapping. Default is False.
#         Can be a `State` object for dynamic updates.
#     disablePortal : State or bool, optional
#         If True, popup is rendered under parent hierarchy. Default is False.
#         Can be a `State` object for dynamic updates.
#     filterOptions : State or Callable, optional
#         Custom function to filter options. Default is None.
#         Can be a `State` object for dynamic updates.
#     filterSelectedOptions : State or bool, optional
#         If True, hides selected options. Default is False.
#         Can be a `State` object for dynamic updates.
#     forcePopupIcon : State or str, optional
#         Controls popup icon visibility ('auto', 'true', 'false'). Default is 'auto'.
#         Can be a `State` object for dynamic updates.
#     freeSolo : State or bool, optional
#         If True, allows free text input. Default is False.
#         Can be a `State` object for dynamic updates.
#     fullWidth : State or bool, optional
#         If True, input takes full width. Default is False.
#         Can be a `State` object for dynamic updates.
#     getLimitTagsText : State or Callable, optional
#         Label when tags are limited. Default is None.
#         Can be a `State` object for dynamic updates.
#     getOptionDisabled : State or Callable, optional
#         Function to disable options. Default is None.
#         Can be a `State` object for dynamic updates.
#     getOptionKey : State or Callable, optional
#         Function to get option key. Default is None.
#         Can be a `State` object for dynamic updates.
#     getOptionLabel : State or Callable, optional
#         Function to get option label. Default is None.
#         Can be a `State` object for dynamic updates.
#     groupBy : State or Callable, optional
#         Function to group options. Default is None.
#         Can be a `State` object for dynamic updates.
#     handleHomeEndKeys : State or bool, optional
#         If True, enables Home/End key navigation. Default is True.
#         Can be a `State` object for dynamic updates.
#     id : State or str, optional
#         ID for the component. Default is None.
#         Can be a `State` object for dynamic updates.
#     includeInputInList : State or bool, optional
#         If True, highlights input in list. Default is False.
#         Can be a `State` object for dynamic updates.
#     inputValue : State or str, optional
#         Current value of the input. Default is None.
#         Can be a `State` object for dynamic updates.
#     isOptionEqualToValue : State or Callable, optional
#         Function to check if option equals value. Default is None.
#         Can be a `State` object for dynamic updates.
#     label : State or str, optional
#         Label for the input. Default is None.
#         Can be a `State` object for dynamic updates.
#     limitTags : State or int, optional
#         Maximum number of tags. Default is -1.
#         Can be a `State` object for dynamic updates.
#     ListboxComponent : State or type, optional
#         Component for listbox. Default is None.
#         Can be a `State` object for dynamic updates.
#     ListboxProps : State or Dict, optional
#         Props for Listbox. Default is None.
#         Can be a `State` object for dynamic updates.
#     loading : State or bool, optional
#         If True, shows loading state. Default is False.
#         Can be a `State` object for dynamic updates.
#     loadingText : State or str, optional
#         Text shown when loading. Default is 'Loading...'.
#         Can be a `State` object for dynamic updates.
#     multiple : State or bool, optional
#         If True, allows multiple selections. Default is False.
#         Can be a `State` object for dynamic updates.
#     name : State or str, optional
#         Name attribute for input. Default is None.
#         Can be a `State` object for dynamic updates.
#     noOptionsText : State or str, optional
#         Text when no options are available. Default is 'No options'.
#         Can be a `State` object for dynamic updates.
#     open : State or bool, optional
#         If True, shows the component. Default is False.
#         Can be a `State` object for dynamic updates.
#     onChange : State or Callable, optional
#         Callback when a menu item is selected. Default is None.
#         Can be a `State` object for dynamic updates.
#         Signature: (event: Any, child: object) -> None
#     openOnFocus : State or bool, optional
#         If True, opens popup on focus. Default is False.
#         Can be a `State` object for dynamic updates.
#     openText : State or str, optional
#         Text for open icon. Default is 'Open'.
#         Can be a `State` object for dynamic updates.
#     PaperComponent : State or type, optional
#         Component for popup body. Default is None.
#         Can be a `State` object for dynamic updates.
#     PopperComponent : State or type, optional
#         Component for popup positioning. Default is None.
#         Can be a `State` object for dynamic updates.
#     popupIcon : State or QWidget, optional
#         Icon for popup. Default is None.
#         Can be a `State` object for dynamic updates.
#     readOnly : State or bool, optional
#         If True, component is read-only. Default is False.
#         Can be a `State` object for dynamic updates.
#     renderOption : State or Callable, optional
#         Function to render each option. Default is None.
#         Can be a `State` object for dynamic updates.
#     renderTags : State or Callable, optional
#         Function to render selected tags. Default is None.
#         Can be a `State` object for dynamic updates.
#     renderValue : State or Callable, optional
#         Function to render selected value. Default is None.
#         Can be a `State` object for dynamic updates.
#     selectOnFocus : State or bool, optional
#         If True, selects input on focus. Default is True.
#         Can be a `State` object for dynamic updates.
#     selected : State or bool, optional
#         If True, highlights the input. Default is False.
#         Can be a `State` object for dynamic updates.
#     size : State or str, optional
#         Size of the component ('small', 'medium', 'large'). Default is 'medium'.
#         Can be a `State` object for dynamic updates.
#     slotProps : State or Dict, optional
#         Props for slots. Default is None.
#         Can be a `State` object for dynamic updates.
#     slots : State or Dict, optional
#         Components for slots. Default is None.
#         Can be a `State` object for dynamic updates.
#     sx : State, list, dict, Callable, str, or None, optional
#         System prop for CSS overrides. Default is None.
#         Can be a `State` object for dynamic updates.
#     value : State or Any, optional
#         The input value. Default is None.
#         Can be a `State` object for dynamic updates.
#     treeView : State or bool, optional
#         If True, enables tree view mode. Default is False.
#         Can be a `State` object for dynamic updates.
#     autoWidth : State or bool, optional
#         If True, auto-adjusts popover width. Default is False.
#         Can be a `State` object for dynamic updates.
#     defaultOpen : State or bool, optional
#         If True, component is initially open. Default is False.
#         Can be a `State` object for dynamic updates.
#     displayEmpty : State or bool, optional
#         If True, displays value when no items selected. Default is False.
#         Can be a `State` object for dynamic updates.
#     IconComponent : State or QWidget, optional
#         Icon for the arrow. Default is None.
#         Can be a `State` object for dynamic updates.
#     input : State or QWidget, optional
#         Input component. Default is None (uses TextField).
#         Can be a `State` object for dynamic updates.
#     inputProps : State or Dict, optional
#         Attributes for input element. Default is None.
#         Can be a `State` object for dynamic updates.
#     labelId : State or str, optional
#         ID for additional label. Default is None.
#         Can be a `State` object for dynamic updates.
#     MenuProps : State or Dict, optional
#         Props for Menu element. Default is None.
#         Can be a `State` object for dynamic updates.
#     native : State or bool, optional
#         If True, uses native select element. Default is False.
#         Can be a `State` object for dynamic updates.
#     SelectDisplayProps : State or Dict, optional
#         Props for clickable div. Default is None.
#         Can be a `State` object for dynamic updates.
#     variant : State or str, optional
#         Variant of the input ('filled', 'outlined', 'standard'). Default is 'outlined'.
#         Can be a `State` object for dynamic updates.
#     **kwargs
#         Additional keyword arguments passed to the parent `QWidget`,
#         supporting props of OutlinedInput.

#     Signals
#     -------
#     onOpen : Signal
#         Emitted when the popup opens.
#     onClose : Signal
#         Emitted when the popup closes.
#     changed : Signal
#         Emitted when the value changes.
#     setupUi : Signal
#         Emitted when UI setup is complete.

#     Notes
#     -----
#     - Existing parameters (53) are retained; 9 new parameters added to align with MUI.
#     - Props of OutlinedInput are supported via `inputProps` and `**kwargs`.
#     - MUI classes applied: `MuiSelect-root`, `Mui-disabled`.
#     - Integrates with `TextField` for input rendering.

#     Demos:
#     - Select: https://qtmui.com/material-ui/qtmui-select/

#     API Reference:
#     - Select API: https://qtmui.com/material-ui/api/select/
#     """

#     onOpen = Signal()
#     onClose = Signal()
#     changed = Signal(object, object)
#     setupUi = Signal()

#     VALID_SIZES = ['small', 'medium', 'large']
#     VALID_VARIANTS = ['filled', 'outlined', 'standard']
#     VALID_FORCE_POPUP_ICON = ['auto', 'true', 'false']

#     def __init__(
#         self,
#         options: Optional[Union[State, List]] = [],
#         renderInput: Optional[Union[State, Callable]] = None,
#         autoComplete: Union[State, bool] = False,
#         autoHighlight: Union[State, bool] = False,
#         autoSelect: Union[State, bool] = False,
#         blurOnSelect: Union[State, bool] = False,
#         ChipProps: Optional[Union[State, Dict]] = None,
#         classes: Optional[Union[State, Dict]] = None,
#         clearIcon: Optional[Union[State, QWidget]] = None,
#         clearOnBlur: Union[State, bool] = False,
#         clearOnEscape: Union[State, bool] = False,
#         clearText: Union[State, str] = "Clear",
#         closeText: Union[State, str] = "Close",
#         children: Optional[Union[State, List[QWidget]]] = None,
#         componentsProps: Optional[Union[State, Dict]] = None,
#         defaultValue: Optional[Union[State, Any]] = None,
#         disableClearable: Union[State, bool] = False,
#         disableCloseOnSelect: Union[State, bool] = False,
#         disabled: Union[State, bool] = False,
#         disabledItemsFocusable: Union[State, bool] = False,
#         disableListWrap: Union[State, bool] = False,
#         disablePortal: Union[State, bool] = False,
#         filterOptions: Optional[Union[State, Callable]] = None,
#         filterSelectedOptions: Union[State, bool] = False,
#         forcePopupIcon: Union[State, str] = "auto",
#         freeSolo: Union[State, bool] = False,
#         fullWidth: Union[State, bool] = False,
#         getLimitTagsText: Optional[Union[State, Callable]] = None,
#         getOptionDisabled: Optional[Union[State, Callable]] = None,
#         getOptionKey: Optional[Union[State, Callable]] = None,
#         getOptionLabel: Optional[Union[State, Callable]] = None,
#         groupBy: Optional[Union[State, Callable]] = None,
#         handleHomeEndKeys: Union[State, bool] = True,
#         id: Optional[Union[State, str]] = None,
#         includeInputInList: Union[State, bool] = False,
#         inputValue: Optional[Union[State, str]] = None,
#         isOptionEqualToValue: Optional[Union[State, Callable]] = None,
#         label: Optional[Union[State, str]] = None,
#         limitTags: Union[State, int] = -1,
#         ListboxComponent: Optional[Union[State, type]] = None,
#         ListboxProps: Optional[Union[State, Dict]] = None,
#         loading: Union[State, bool] = False,
#         loadingText: Union[State, str] = "Loading...",
#         multiple: Union[State, bool] = False,
#         name: Optional[Union[State, str]] = None,
#         noOptionsText: Union[State, str] = "No options",
#         open: Union[State, bool] = False,
#         onChange: Optional[Union[State, Callable]] = None,
#         openOnFocus: Union[State, bool] = False,
#         openText: Union[State, str] = "Open",
#         PaperComponent: Optional[Union[State, type]] = None,
#         PopperComponent: Optional[Union[State, type]] = None,
#         popupIcon: Optional[Union[State, QWidget]] = None,
#         readOnly: Union[State, bool] = False,
#         renderOption: Optional[Union[State, Callable]] = None,
#         renderTags: Optional[Union[State, Callable]] = None,
#         renderValue: Optional[Union[State, Callable]] = None,
#         selectOnFocus: Union[State, bool] = True,
#         selected: Union[State, bool] = False,
#         size: Union[State, str] = "medium",
#         slotProps: Optional[Union[State, Dict]] = None,
#         slots: Optional[Union[State, Dict]] = None,
#         sx: Optional[Union[State, List, Dict, Callable, str]] = None,
#         value: Optional[Union[State, Any]] = None,
#         treeView: Union[State, bool] = False,
#         autoWidth: Union[State, bool] = False,
#         defaultOpen: Union[State, bool] = False,
#         displayEmpty: Union[State, bool] = False,
#         IconComponent: Optional[Union[State, QWidget]] = None,
#         input: Optional[Union[State, QWidget]] = None,
#         inputProps: Optional[Union[State, Dict]] = None,
#         labelId: Optional[Union[State, str]] = None,
#         MenuProps: Optional[Union[State, Dict]] = None,
#         native: Union[State, bool] = False,
#         SelectDisplayProps: Optional[Union[State, Dict]] = None,
#         variant: Union[State, str] = "outlined",
#         **kwargs
#     ):
#         super().__init__(**kwargs)
#         self.theme = useTheme()
#         self._widget_references = []

#         # Set properties with validation
#         self._set_options(options)
#         self._set_renderInput(renderInput)
#         self._set_autoComplete(autoComplete)
#         self._set_autoHighlight(autoHighlight)
#         self._set_autoSelect(autoSelect)
#         self._set_blurOnSelect(blurOnSelect)
#         self._set_ChipProps(ChipProps)
#         self._set_classes(classes)
#         self._set_clearIcon(clearIcon)
#         self._set_clearOnBlur(clearOnBlur)
#         self._set_clearOnEscape(clearOnEscape)
#         self._set_clearText(clearText)
#         self._set_closeText(closeText)
#         self._set_children(children)
#         self._set_componentsProps(componentsProps)
#         self._set_defaultValue(defaultValue)
#         self._set_disableClearable(disableClearable)
#         self._set_disableCloseOnSelect(disableCloseOnSelect)
#         self._set_disabled(disabled)
#         self._set_disabledItemsFocusable(disabledItemsFocusable)
#         self._set_disableListWrap(disableListWrap)
#         self._set_disablePortal(disablePortal)
#         self._set_filterOptions(filterOptions)
#         self._set_filterSelectedOptions(filterSelectedOptions)
#         self._set_forcePopupIcon(forcePopupIcon)
#         self._set_freeSolo(freeSolo)
#         self._set_fullWidth(fullWidth)
#         self._set_getLimitTagsText(getLimitTagsText)
#         self._set_getOptionDisabled(getOptionDisabled)
#         self._set_getOptionKey(getOptionKey)
#         self._set_getOptionLabel(getOptionLabel)
#         self._set_groupBy(groupBy)
#         self._set_handleHomeEndKeys(handleHomeEndKeys)
#         self._set_id(id)
#         self._set_includeInputInList(includeInputInList)
#         self._set_inputValue(inputValue)
#         self._set_isOptionEqualToValue(isOptionEqualToValue)
#         self._set_label(label)
#         self._set_limitTags(limitTags)
#         self._set_ListboxComponent(ListboxComponent)
#         self._set_ListboxProps(ListboxProps)
#         self._set_loading(loading)
#         self._set_loadingText(loadingText)
#         self._set_multiple(multiple)
#         self._set_name(name)
#         self._set_noOptionsText(noOptionsText)
#         self._set_open(open)
#         self._set_onChange(onChange)
#         self._set_openOnFocus(openOnFocus)
#         self._set_openText(openText)
#         self._set_PaperComponent(PaperComponent)
#         self._set_PopperComponent(PopperComponent)
#         self._set_popupIcon(popupIcon)
#         self._set_readOnly(readOnly)
#         self._set_renderOption(renderOption)
#         self._set_renderTags(renderTags)
#         self._set_renderValue(renderValue)
#         self._set_selectOnFocus(selectOnFocus)
#         self._set_selected(selected)
#         self._set_size(size)
#         self._set_slotProps(slotProps)
#         self._set_slots(slots)
#         self._set_sx(sx)
#         self._set_value(value)
#         self._set_treeView(treeView)
#         self._set_autoWidth(autoWidth)
#         self._set_defaultOpen(defaultOpen)
#         self._set_displayEmpty(displayEmpty)
#         self._set_IconComponent(IconComponent)
#         self._set_input(input)
#         self._set_inputProps(inputProps)
#         self._set_labelId(labelId)
#         self._set_MenuProps(MenuProps)
#         self._set_native(native)
#         self._set_SelectDisplayProps(SelectDisplayProps)
#         self._set_variant(variant)

#         self._state = self._value
#         self._init_ui()
#         self._set_stylesheet()

#         self.useEffect(
            self._set_stylesheet,
            [theme.state]
        )
#         self.destroyed.connect(self._on_destroyed)
#         self._connect_signals()

#     # Setter and Getter methods
#     @_validate_param(file_path="qtmui.material.select", param_name="options", supported_signatures=Union[State, List, type(None)])
#     def _set_options(self, value):
#         """Assign value to options."""
#         self._options = value

#     def _get_options(self):
#         """Get the options value."""
#         return self._options.value if isinstance(self._options, State) else self._options

#     @_validate_param(file_path="qtmui.material.select", param_name="renderInput", supported_signatures=Union[State, Callable, type(None)])
#     def _set_renderInput(self, value):
#         """Assign value to renderInput."""
#         self._renderInput = value

#     def _get_renderInput(self):
#         """Get the renderInput value."""
#         return self._renderInput.value if isinstance(self._renderInput, State) else self._renderInput

#     @_validate_param(file_path="qtmui.material.select", param_name="autoComplete", supported_signatures=Union[State, bool])
#     def _set_autoComplete(self, value):
#         """Assign value to autoComplete."""
#         self._autoComplete = value

#     def _get_autoComplete(self):
#         """Get the autoComplete value."""
#         return self._autoComplete.value if isinstance(self._autoComplete, State) else self._autoComplete

#     @_validate_param(file_path="qtmui.material.select", param_name="autoHighlight", supported_signatures=Union[State, bool])
#     def _set_autoHighlight(self, value):
#         """Assign value to autoHighlight."""
#         self._autoHighlight = value

#     def _get_autoHighlight(self):
#         """Get the autoHighlight value."""
#         return self._autoHighlight.value if isinstance(self._autoHighlight, State) else self._autoHighlight

#     @_validate_param(file_path="qtmui.material.select", param_name="autoSelect", supported_signatures=Union[State, bool])
#     def _set_autoSelect(self, value):
#         """Assign value to autoSelect."""
#         self._autoSelect = value

#     def _get_autoSelect(self):
#         """Get the autoSelect value."""
#         return self._autoSelect.value if isinstance(self._autoSelect, State) else self._autoSelect

#     @_validate_param(file_path="qtmui.material.select", param_name="blurOnSelect", supported_signatures=Union[State, bool])
#     def _set_blurOnSelect(self, value):
#         """Assign value to blurOnSelect."""
#         self._blurOnSelect = value

#     def _get_blurOnSelect(self):
#         """Get the blurOnSelect value."""
#         return self._blurOnSelect.value if isinstance(self._blurOnSelect, State) else self._blurOnSelect

#     @_validate_param(file_path="qtmui.material.select", param_name="ChipProps", supported_signatures=Union[State, Dict, type(None)])
#     def _set_ChipProps(self, value):
#         """Assign value to ChipProps."""
#         self._ChipProps = value

#     def _get_ChipProps(self):
#         """Get the ChipProps value."""
#         return self._ChipProps.value if isinstance(self._ChipProps, State) else self._ChipProps

#     @_validate_param(file_path="qtmui.material.select", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
#     def _set_classes(self, value):
#         """Assign value to classes."""
#         self._classes = value

#     def _get_classes(self):
#         """Get the classes value."""
#         return self._classes.value if isinstance(self._classes, State) else self._classes

#     @_validate_param(file_path="qtmui.material.select", param_name="clearIcon", supported_signatures=Union[State, QWidget, type(None)])
#     def _set_clearIcon(self, value):
#         """Assign value to clearIcon."""
#         self._clearIcon = value

#     def _get_clearIcon(self):
#         """Get the clearIcon value."""
#         return self._clearIcon.value if isinstance(self._clearIcon, State) else self._clearIcon

#     @_validate_param(file_path="qtmui.material.select", param_name="clearOnBlur", supported_signatures=Union[State, bool])
#     def _set_clearOnBlur(self, value):
#         """Assign value to clearOnBlur."""
#         self._clearOnBlur = value

#     def _get_clearOnBlur(self):
#         """Get the clearOnBlur value."""
#         return self._clearOnBlur.value if isinstance(self._clearOnBlur, State) else self._clearOnBlur

#     @_validate_param(file_path="qtmui.material.select", param_name="clearOnEscape", supported_signatures=Union[State, bool])
#     def _set_clearOnEscape(self, value):
#         """Assign value to clearOnEscape."""
#         self._clearOnEscape = value

#     def _get_clearOnEscape(self):
#         """Get the clearOnEscape value."""
#         return self._clearOnEscape.value if isinstance(self._clearOnEscape, State) else self._clearOnEscape

#     @_validate_param(file_path="qtmui.material.select", param_name="clearText", supported_signatures=Union[State, str])
#     def _set_clearText(self, value):
#         """Assign value to clearText."""
#         self._clearText = value

#     def _get_clearText(self):
#         """Get the clearText value."""
#         return self._clearText.value if isinstance(self._clearText, State) else self._clearText

#     @_validate_param(file_path="qtmui.material.select", param_name="closeText", supported_signatures=Union[State, str])
#     def _set_closeText(self, value):
#         """Assign value to closeText."""
#         self._closeText = value

#     def _get_closeText(self):
#         """Get the closeText value."""
#         return self._closeText.value if isinstance(self._closeText, State) else self._closeText

#     @_validate_param(file_path="qtmui.material.select", param_name="children", supported_signatures=Union[State, List[QWidget], type(None)])
#     def _set_children(self, value):
#         """Assign value to children."""
#         self._children = value
#         children = value.value if isinstance(value, State) else value
#         if isinstance(children, list):
#             for child in children:
#                 if not isinstance(child, QWidget):
#                     raise TypeError(f"Each element in children must be a QWidget, got {type(child)}")
#                 self._widget_references.append(child)

#     def _get_children(self):
#         """Get the children value."""
#         return self._children.value if isinstance(self._children, State) else self._children

#     @_validate_param(file_path="qtmui.material.select", param_name="componentsProps", supported_signatures=Union[State, Dict, type(None)])
#     def _set_componentsProps(self, value):
#         """Assign value to componentsProps."""
#         self._componentsProps = value

#     def _get_componentsProps(self):
#         """Get the componentsProps value."""
#         return self._componentsProps.value if isinstance(self._componentsProps, State) else self._componentsProps

#     @_validate_param(file_path="qtmui.material.select", param_name="defaultValue", supported_signatures=Union[State, Any, type(None)])
#     def _set_defaultValue(self, value):
#         """Assign value to defaultValue."""
#         self._defaultValue = value

#     def _get_defaultValue(self):
#         """Get the defaultValue value."""
#         return self._defaultValue.value if isinstance(self._defaultValue, State) else self._defaultValue

#     @_validate_param(file_path="qtmui.material.select", param_name="disableClearable", supported_signatures=Union[State, bool])
#     def _set_disableClearable(self, value):
#         """Assign value to disableClearable."""
#         self._disableClearable = value

#     def _get_disableClearable(self):
#         """Get the disableClearable value."""
#         return self._disableClearable.value if isinstance(self._disableClearable, State) else self._disableClearable

#     @_validate_param(file_path="qtmui.material.select", param_name="disableCloseOnSelect", supported_signatures=Union[State, bool])
#     def _set_disableCloseOnSelect(self, value):
#         """Assign value to disableCloseOnSelect."""
#         self._disableCloseOnSelect = value

#     def _get_disableCloseOnSelect(self):
#         """Get the disableCloseOnSelect value."""
#         return self._disableCloseOnSelect.value if isinstance(self._disableCloseOnSelect, State) else self._disableCloseOnSelect

#     @_validate_param(file_path="qtmui.material.select", param_name="disabled", supported_signatures=Union[State, bool])
#     def _set_disabled(self, value):
#         """Assign value to disabled."""
#         self._disabled = value

#     def _get_disabled(self):
#         """Get the disabled value."""
#         return self._disabled.value if isinstance(self._disabled, State) else self._disabled

#     @_validate_param(file_path="qtmui.material.select", param_name="disabledItemsFocusable", supported_signatures=Union[State, bool])
#     def _set_disabledItemsFocusable(self, value):
#         """Assign value to disabledItemsFocusable."""
#         self._disabledItemsFocusable = value

#     def _get_disabledItemsFocusable(self):
#         """Get the disabledItemsFocusable value."""
#         return self._disabledItemsFocusable.value if isinstance(self._disabledItemsFocusable, State) else self._disabledItemsFocusable

#     @_validate_param(file_path="qtmui.material.select", param_name="disableListWrap", supported_signatures=Union[State, bool])
#     def _set_disableListWrap(self, value):
#         """Assign value to disableListWrap."""
#         self._disableListWrap = value

#     def _get_disableListWrap(self):
#         """Get the disableListWrap value."""
#         return self._disableListWrap.value if isinstance(self._disableListWrap, State) else self._disableListWrap

#     @_validate_param(file_path="qtmui.material.select", param_name="disablePortal", supported_signatures=Union[State, bool])
#     def _set_disablePortal(self, value):
#         """Assign value to disablePortal."""
#         self._disablePortal = value

#     def _get_disablePortal(self):
#         """Get the disablePortal value."""
#         return self._disablePortal.value if isinstance(self._disablePortal, State) else self._disablePortal

#     @_validate_param(file_path="qtmui.material.select", param_name="filterOptions", supported_signatures=Union[State, Callable, type(None)])
#     def _set_filterOptions(self, value):
#         """Assign value to filterOptions."""
#         self._filterOptions = value

#     def _get_filterOptions(self):
#         """Get the filterOptions value."""
#         return self._filterOptions.value if isinstance(self._filterOptions, State) else self._filterOptions

#     @_validate_param(file_path="qtmui.material.select", param_name="filterSelectedOptions", supported_signatures=Union[State, bool])
#     def _set_filterSelectedOptions(self, value):
#         """Assign value to filterSelectedOptions."""
#         self._filterSelectedOptions = value

#     def _get_filterSelectedOptions(self):
#         """Get the filterSelectedOptions value."""
#         return self._filterSelectedOptions.value if isinstance(self._filterSelectedOptions, State) else self._filterSelectedOptions

#     @_validate_param(file_path="qtmui.material.select", param_name="forcePopupIcon", supported_signatures=Union[State, str], valid_values=VALID_FORCE_POPUP_ICON)
#     def _set_forcePopupIcon(self, value):
#         """Assign value to forcePopupIcon."""
#         self._forcePopupIcon = value

#     def _get_forcePopupIcon(self):
#         """Get the forcePopupIcon value."""
#         return self._forcePopupIcon.value if isinstance(self._forcePopupIcon, State) else self._forcePopupIcon

#     @_validate_param(file_path="qtmui.material.select", param_name="freeSolo", supported_signatures=Union[State, bool])
#     def _set_freeSolo(self, value):
#         """Assign value to freeSolo."""
#         self._freeSolo = value

#     def _get_freeSolo(self):
#         """Get the freeSolo value."""
#         return self._freeSolo.value if isinstance(self._freeSolo, State) else self._freeSolo

#     @_validate_param(file_path="qtmui.material.select", param_name="fullWidth", supported_signatures=Union[State, bool])
#     def _set_fullWidth(self, value):
#         """Assign value to fullWidth."""
#         self._fullWidth = value

#     def _get_fullWidth(self):
#         """Get the fullWidth value."""
#         return self._fullWidth.value if isinstance(self._fullWidth, State) else self._fullWidth

#     @_validate_param(file_path="qtmui.material.select", param_name="getLimitTagsText", supported_signatures=Union[State, Callable, type(None)])
#     def _set_getLimitTagsText(self, value):
#         """Assign value to getLimitTagsText."""
#         self._getLimitTagsText = value

#     def _get_getLimitTagsText(self):
#         """Get the getLimitTagsText value."""
#         return self._getLimitTagsText.value if isinstance(self._getLimitTagsText, State) else self._getLimitTagsText

#     @_validate_param(file_path="qtmui.material.select", param_name="getOptionDisabled", supported_signatures=Union[State, Callable, type(None)])
#     def _set_getOptionDisabled(self, value):
#         """Assign value to getOptionDisabled."""
#         self._getOptionDisabled = value

#     def _get_getOptionDisabled(self):
#         """Get the getOptionDisabled value."""
#         return self._getOptionDisabled.value if isinstance(self._getOptionDisabled, State) else self._getOptionDisabled

#     @_validate_param(file_path="qtmui.material.select", param_name="getOptionKey", supported_signatures=Union[State, Callable, type(None)])
#     def _set_getOptionKey(self, value):
#         """Assign value to getOptionKey."""
#         self._getOptionKey = value

#     def _get_getOptionKey(self):
#         """Get the getOptionKey value."""
#         return self._getOptionKey.value if isinstance(self._getOptionKey, State) else self._getOptionKey

#     @_validate_param(file_path="qtmui.material.select", param_name="getOptionLabel", supported_signatures=Union[State, Callable, type(None)])
#     def _set_getOptionLabel(self, value):
#         """Assign value to getOptionLabel."""
#         self._getOptionLabel = value

#     def _get_getOptionLabel(self):
#         """Get the getOptionLabel value."""
#         return self._getOptionLabel.value if isinstance(self._getOptionLabel, State) else self._getOptionLabel

#     @_validate_param(file_path="qtmui.material.select", param_name="groupBy", supported_signatures=Union[State, Callable, type(None)])
#     def _set_groupBy(self, value):
#         """Assign value to groupBy."""
#         self._groupBy = value

#     def _get_groupBy(self):
#         """Get the groupBy value."""
#         return self._groupBy.value if isinstance(self._groupBy, State) else self._groupBy

#     @_validate_param(file_path="qtmui.material.select", param_name="handleHomeEndKeys", supported_signatures=Union[State, bool])
#     def _set_handleHomeEndKeys(self, value):
#         """Assign value to handleHomeEndKeys."""
#         self._handleHomeEndKeys = value

#     def _get_handleHomeEndKeys(self):
#         """Get the handleHomeEndKeys value."""
#         return self._handleHomeEndKeys.value if isinstance(self._handleHomeEndKeys, State) else self._handleHomeEndKeys

#     @_validate_param(file_path="qtmui.material.select", param_name="id", supported_signatures=Union[State, str, type(None)])
#     def _set_id(self, value):
#         """Assign value to id."""
#         self._id = value

#     def _get_id(self):
#         """Get the id value."""
#         return self._id.value if isinstance(self._id, State) else self._id

#     @_validate_param(file_path="qtmui.material.select", param_name="includeInputInList", supported_signatures=Union[State, bool])
#     def _set_includeInputInList(self, value):
#         """Assign value to includeInputInList."""
#         self._includeInputInList = value

#     def _get_includeInputInList(self):
#         """Get the includeInputInList value."""
#         return self._includeInputInList.value if isinstance(self._includeInputInList, State) else self._includeInputInList

#     @_validate_param(file_path="qtmui.material.select", param_name="inputValue", supported_signatures=Union[State, str, type(None)])
#     def _set_inputValue(self, value):
#         """Assign value to inputValue."""
#         self._inputValue = value

#     def _get_inputValue(self):
#         """Get the inputValue value."""
#         return self._inputValue.value if isinstance(self._inputValue, State) else self._inputValue

#     @_validate_param(file_path="qtmui.material.select", param_name="isOptionEqualToValue", supported_signatures=Union[State, Callable, type(None)])
#     def _set_isOptionEqualToValue(self, value):
#         """Assign value to isOptionEqualToValue."""
#         self._isOptionEqualToValue = value

#     def _get_isOptionEqualToValue(self):
#         """Get the isOptionEqualToValue value."""
#         return self._isOptionEqualToValue.value if isinstance(self._isOptionEqualToValue, State) else self._isOptionEqualToValue

#     @_validate_param(file_path="qtmui.material.select", param_name="label", supported_signatures=Union[State, str, type(None)])
#     def _set_label(self, value):
#         """Assign value to label."""
#         self._label = value

#     def _get_label(self):
#         """Get the label value."""
#         return self._label.value if isinstance(self._label, State) else self._label

#     @_validate_param(file_path="qtmui.material.select", param_name="limitTags", supported_signatures=Union[State, int])
#     def _set_limitTags(self, value):
#         """Assign value to limitTags."""
#         self._limitTags = value

#     def _get_limitTags(self):
#         """Get the limitTags value."""
#         return self._limitTags.value if isinstance(self._limitTags, State) else self._limitTags

#     @_validate_param(file_path="qtmui.material.select", param_name="ListboxComponent", supported_signatures=Union[State, type, type(None)])
#     def _set_ListboxComponent(self, value):
#         """Assign value to ListboxComponent."""
#         self._ListboxComponent = value

#     def _get_ListboxComponent(self):
#         """Get the ListboxComponent value."""
#         return self._ListboxComponent.value if isinstance(self._ListboxComponent, State) else self._ListboxComponent

#     @_validate_param(file_path="qtmui.material.select", param_name="ListboxProps", supported_signatures=Union[State, Dict, type(None)])
#     def _set_ListboxProps(self, value):
#         """Assign value to ListboxProps."""
#         self._ListboxProps = value

#     def _get_ListboxProps(self):
#         """Get the ListboxProps value."""
#         return self._ListboxProps.value if isinstance(self._ListboxProps, State) else self._ListboxProps

#     @_validate_param(file_path="qtmui.material.select", param_name="loading", supported_signatures=Union[State, bool])
#     def _set_loading(self, value):
#         """Assign value to loading."""
#         self._loading = value

#     def _get_loading(self):
#         """Get the loading value."""
#         return self._loading.value if isinstance(self._loading, State) else self._loading

#     @_validate_param(file_path="qtmui.material.select", param_name="loadingText", supported_signatures=Union[State, str])
#     def _set_loadingText(self, value):
#         """Assign value to loadingText."""
#         self._loadingText = value

#     def _get_loadingText(self):
#         """Get the loadingText value."""
#         return self._loadingText.value if isinstance(self._loadingText, State) else self._loadingText

#     @_validate_param(file_path="qtmui.material.select", param_name="multiple", supported_signatures=Union[State, bool])
#     def _set_multiple(self, value):
#         """Assign value to multiple."""
#         self._multiple = value

#     def _get_multiple(self):
#         """Get the multiple value."""
#         return self._multiple.value if isinstance(self._multiple, State) else self._multiple

#     @_validate_param(file_path="qtmui.material.select", param_name="name", supported_signatures=Union[State, str, type(None)])
#     def _set_name(self, value):
#         """Assign value to name."""
#         self._name = value

#     def _get_name(self):
#         """Get the name value."""
#         return self._name.value if isinstance(self._name, State) else self._name

#     @_validate_param(file_path="qtmui.material.select", param_name="noOptionsText", supported_signatures=Union[State, str])
#     def _set_noOptionsText(self, value):
#         """Assign value to noOptionsText."""
#         self._noOptionsText = value

#     def _get_noOptionsText(self):
#         """Get the noOptionsText value."""
#         return self._noOptionsText.value if isinstance(self._noOptionsText, State) else self._noOptionsText

#     @_validate_param(file_path="qtmui.material.select", param_name="open", supported_signatures=Union[State, bool])
#     def _set_open(self, value):
#         """Assign value to open."""
#         self._open = value

#     def _get_open(self):
#         """Get the open value."""
#         return self._open.value if isinstance(self._open, State) else self._open

#     @_validate_param(file_path="qtmui.material.select", param_name="onChange", supported_signatures=Union[State, Callable, type(None)])
#     def _set_onChange(self, value):
#         """Assign value to onChange."""
#         self._onChange = value

#     def _get_onChange(self):
#         """Get the onChange value."""
#         return self._onChange.value if isinstance(self._onChange, State) else self._onChange

#     @_validate_param(file_path="qtmui.material.select", param_name="openOnFocus", supported_signatures=Union[State, bool])
#     def _set_openOnFocus(self, value):
#         """Assign value to openOnFocus."""
#         self._openOnFocus = value

#     def _get_openOnFocus(self):
#         """Get the openOnFocus value."""
#         return self._openOnFocus.value if isinstance(self._openOnFocus, State) else self._openOnFocus

#     @_validate_param(file_path="qtmui.material.select", param_name="openText", supported_signatures=Union[State, str])
#     def _set_openText(self, value):
#         """Assign value to openText."""
#         self._openText = value

#     def _get_openText(self):
#         """Get the openText value."""
#         return self._openText.value if isinstance(self._openText, State) else self._openText

#     @_validate_param(file_path="qtmui.material.select", param_name="PaperComponent", supported_signatures=Union[State, type, type(None)])
#     def _set_PaperComponent(self, value):
#         """Assign value to PaperComponent."""
#         self._PaperComponent = value

#     def _get_PaperComponent(self):
#         """Get the PaperComponent value."""
#         return self._PaperComponent.value if isinstance(self._PaperComponent, State) else self._PaperComponent

#     @_validate_param(file_path="qtmui.material.select", param_name="PopperComponent", supported_signatures=Union[State, type, type(None)])
#     def _set_PopperComponent(self, value):
#         """Assign value to PopperComponent."""
#         self._PopperComponent = value

#     def _get_PopperComponent(self):
#         """Get the PopperComponent value."""
#         return self._PopperComponent.value if isinstance(self._PopperComponent, State) else self._PopperComponent

#     @_validate_param(file_path="qtmui.material.select", param_name="popupIcon", supported_signatures=Union[State, QWidget, type(None)])
#     def _set_popupIcon(self, value):
#         """Assign value to popupIcon."""
#         self._popupIcon = value

#     def _get_popupIcon(self):
#         """Get the popupIcon value."""
#         return self._popupIcon.value if isinstance(self._popupIcon, State) else self._popupIcon

#     @_validate_param(file_path="qtmui.material.select", param_name="readOnly", supported_signatures=Union[State, bool])
#     def _set_readOnly(self, value):
#         """Assign value to readOnly."""
#         self._readOnly = value

#     def _get_readOnly(self):
#         """Get the readOnly value."""
#         return self._readOnly.value if isinstance(self._readOnly, State) else self._readOnly

#     @_validate_param(file_path="qtmui.material.select", param_name="renderOption", supported_signatures=Union[State, Callable, type(None)])
#     def _set_renderOption(self, value):
#         """Assign value to renderOption."""
#         self._renderOption = value

#     def _get_renderOption(self):
#         """Get the renderOption value."""
#         return self._renderOption.value if isinstance(self._renderOption, State) else self._renderOption

#     @_validate_param(file_path="qtmui.material.select", param_name="renderTags", supported_signatures=Union[State, Callable, type(None)])
#     def _set_renderTags(self, value):
#         """Assign value to renderTags."""
#         self._renderTags = value

#     def _get_renderTags(self):
#         """Get the renderTags value."""
#         return self._renderTags.value if isinstance(self._renderTags, State) else self._renderTags

#     @_validate_param(file_path="qtmui.material.select", param_name="renderValue", supported_signatures=Union[State, Callable, type(None)])
#     def _set_renderValue(self, value):
#         """Assign value to renderValue."""
#         self._renderValue = value

#     def _get_renderValue(self):
#         """Get the renderValue value."""
#         return self._renderValue.value if isinstance(self._renderValue, State) else self._renderValue

#     @_validate_param(file_path="qtmui.material.select", param_name="selectOnFocus", supported_signatures=Union[State, bool])
#     def _set_selectOnFocus(self, value):
#         """Assign value to selectOnFocus."""
#         self._selectOnFocus = value

#     def _get_selectOnFocus(self):
#         """Get the selectOnFocus value."""
#         return self._selectOnFocus.value if isinstance(self._selectOnFocus, State) else self._selectOnFocus

#     @_validate_param(file_path="qtmui.material.select", param_name="selected", supported_signatures=Union[State, bool])
#     def _set_selected(self, value):
#         """Assign value to selected."""
#         self._selected = value

#     def _get_selected(self):
#         """Get the selected value."""
#         return self._selected.value if isinstance(self._selected, State) else self._selected

#     @_validate_param(file_path="qtmui.material.select", param_name="size", supported_signatures=Union[State, str], valid_values=VALID_SIZES)
#     def _set_size(self, value):
#         """Assign value to size."""
#         self._size = value

#     def _get_size(self):
#         """Get the size value."""
#         return self._size.value if isinstance(self._size, State) else self._size

#     @_validate_param(file_path="qtmui.material.select", param_name="slotProps", supported_signatures=Union[State, Dict, type(None)])
#     def _set_slotProps(self, value):
#         """Assign value to slotProps."""
#         self._slotProps = value

#     def _get_slotProps(self):
#         """Get the slotProps value."""
#         return self._slotProps.value if isinstance(self._slotProps, State) else self._slotProps

#     @_validate_param(file_path="qtmui.material.select", param_name="slots", supported_signatures=Union[State, Dict, type(None)])
#     def _set_slots(self, value):
#         """Assign value to slots."""
#         self._slots = value

#     def _get_slots(self):
#         """Get the slots value."""
#         return self._slots.value if isinstance(self._slots, State) else self._slots

#     @_validate_param(file_path="qtmui.material.select", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, str, type(None)])
#     def _set_sx(self, value):
#         """Assign value to sx."""
#         self._sx = value

#     def _get_sx(self):
#         """Get the sx value."""
#         return self._sx.value if isinstance(self._sx, State) else self._sx

#     @_validate_param(file_path="qtmui.material.select", param_name="value", supported_signatures=Union[State, Any, type(None)])
#     def _set_value(self, value):
#         """Assign value to value."""
#         self._value = value
#         self._state = value

#     def _get_value(self):
#         """Get the value value."""
#         return self._value.value if isinstance(self._value, State) else self._value

#     @_validate_param(file_path="qtmui.material.select", param_name="treeView", supported_signatures=Union[State, bool])
#     def _set_treeView(self, value):
#         """Assign value to treeView."""
#         self._treeView = value

#     def _get_treeView(self):
#         """Get the treeView value."""
#         return self._treeView.value if isinstance(self._treeView, State) else self._treeView

#     @_validate_param(file_path="qtmui.material.select", param_name="autoWidth", supported_signatures=Union[State, bool])
#     def _set_autoWidth(self, value):
#         """Assign value to autoWidth."""
#         self._autoWidth = value

#     def _get_autoWidth(self):
#         """Get the autoWidth value."""
#         return self._autoWidth.value if isinstance(self._autoWidth, State) else self._autoWidth

#     @_validate_param(file_path="qtmui.material.select", param_name="defaultOpen", supported_signatures=Union[State, bool])
#     def _set_defaultOpen(self, value):
#         """Assign value to defaultOpen."""
#         self._defaultOpen = value

#     def _get_defaultOpen(self):
#         """Get the defaultOpen value."""
#         return self._defaultOpen.value if isinstance(self._defaultOpen, State) else self._defaultOpen

#     @_validate_param(file_path="qtmui.material.select", param_name="displayEmpty", supported_signatures=Union[State, bool])
#     def _set_displayEmpty(self, value):
#         """Assign value to displayEmpty."""
#         self._displayEmpty = value

#     def _get_displayEmpty(self):
#         """Get the displayEmpty value."""
#         return self._displayEmpty.value if isinstance(self._displayEmpty, State) else self._displayEmpty

#     @_validate_param(file_path="qtmui.material.select", param_name="IconComponent", supported_signatures=Union[State, QWidget, type(None)])
#     def _set_IconComponent(self, value):
#         """Assign value to IconComponent."""
#         self._IconComponent = value

#     def _get_IconComponent(self):
#         """Get the IconComponent value."""
#         return self._IconComponent.value if isinstance(self._IconComponent, State) else self._IconComponent

#     @_validate_param(file_path="qtmui.material.select", param_name="input", supported_signatures=Union[State, QWidget, type(None)])
#     def _set_input(self, value):
#         """Assign value to input."""
#         self._input = value

#     def _get_input(self):
#         """Get the input value."""
#         return self._input.value if isinstance(self._input, State) else self._input

#     @_validate_param(file_path="qtmui.material.select", param_name="inputProps", supported_signatures=Union[State, Dict, type(None)])
#     def _set_inputProps(self, value):
#         """Assign value to inputProps."""
#         self._inputProps = value

#     def _get_inputProps(self):
#         """Get the inputProps value."""
#         return self._inputProps.value if isinstance(self._inputProps, State) else self._inputProps

#     @_validate_param(file_path="qtmui.material.select", param_name="labelId", supported_signatures=Union[State, str, type(None)])
#     def _set_labelId(self, value):
#         """Assign value to labelId."""
#         self._labelId = value

#     def _get_labelId(self):
#         """Get the labelId value."""
#         return self._labelId.value if isinstance(self._labelId, State) else self._labelId

#     @_validate_param(file_path="qtmui.material.select", param_name="MenuProps", supported_signatures=Union[State, Dict, type(None)])
#     def _set_MenuProps(self, value):
#         """Assign value to MenuProps."""
#         self._MenuProps = value

#     def _get_MenuProps(self):
#         """Get the MenuProps value."""
#         return self._MenuProps.value if isinstance(self._MenuProps, State) else self._MenuProps

#     @_validate_param(file_path="qtmui.material.select", param_name="native", supported_signatures=Union[State, bool])
#     def _set_native(self, value):
#         """Assign value to native."""
#         self._native = value

#     def _get_native(self):
#         """Get the native value."""
#         return self._native.value if isinstance(self._native, State) else self._native

#     @_validate_param(file_path="qtmui.material.select", param_name="SelectDisplayProps", supported_signatures=Union[State, Dict, type(None)])
#     def _set_SelectDisplayProps(self, value):
#         """Assign value to SelectDisplayProps."""
#         self._SelectDisplayProps = value

#     def _get_SelectDisplayProps(self):
#         """Get the SelectDisplayProps value."""
#         return self._SelectDisplayProps.value if isinstance(self._SelectDisplayProps, State) else self._SelectDisplayProps

#     @_validate_param(file_path="qtmui.material.select", param_name="variant", supported_signatures=Union[State, str], valid_values=VALID_VARIANTS)
#     def _set_variant(self, value):
#         """Assign value to variant."""
#         self._variant = value

#     def _get_variant(self):
#         """Get the variant value."""
#         return self._variant.value if isinstance(self._variant, State) else self._variant

#     def _init_ui(self):
#         """Initialize the UI based on props."""
#         self.setLayout(QVBoxLayout())
#         self.layout().setContentsMargins(0, 0, 0, 0)
#         self.layout().setSpacing(self.theme.spacing(1))

#         # Clear previous widgets
#         self._widget_references.clear()
#         while self.layout().count():
#             item = self.layout().takeAt(0)
#             if item.widget():
#                 item.widget().setParent(None)

#         # Determine initial value
#         initial_value = self._get_value() if self._get_value() is not None else self._get_defaultValue()

#         # Create input field
#         input_params = {
#             "label": self._get_label(),
#             "select": True,
#             "autoComplete": self._get_autoComplete(),
#             "disabled": self._get_disabled(),
#             "defaultValue": initial_value,
#             "fullWidth": self._get_fullWidth(),
#             "id": self._get_id(),
#             "options": self._get_options(),
#             "onChange": self._on_input_change,
#             "treeView": self._get_treeView(),
#             "multiple": self._get_multiple(),
#             "getOptionLabel": self._get_getOptionLabel(),
#             "renderOption": self._get_renderOption(),
#             "renderTags": self._get_renderTags(),
#             "renderValue": self._get_renderValue(),
#             "size": self._get_size(),
#             "value": initial_value,
#             "variant": self._get_variant(),
#             **(self._get_inputProps() or {})
#         }
#         self._inputField = self._get_input() or (self._get_renderInput()(input_params) if self._get_renderInput() else TextField(**input_params))
#         self.layout().addWidget(self._inputField)
#         self._widget_references.append(self._inputField)

#         # Set initial open state
#         if self._get_defaultOpen() and not self._get_open():
#             self._set_open(True)
#             self.onOpen.emit()

#         # Start timer to check input field setup
#         QTimer.singleShot(0, self._check_input_field)

#     def _check_input_field(self):
#         """Check if input field is ready and emit setupUi signal."""
#         if hasattr(self._inputField, "_lineEdit"):
#             self.setupUi.emit()
#         else:
#             QTimer.singleShot(200, self._check_input_field)

#     def _setup_ui(self):
#         """Setup UI after input field is ready."""
#         if isinstance(self._inputField, TextField):
#             if isinstance(self._state, State):
#                 self._inputField._lineEdit.setText(self._state.value or "")
#                 self._state.valueChanged.connect(self._update_input_text)
#             elif self._state is not None:
#                 self._inputField._lineEdit.setText(str(self._state))

#         if self._get_selected():
#             self._apply_selected_style()

#         if hasattr(self._inputField, "valueChanged"):
#             self._inputField.valueChanged.connect(self._on_input_change)

#     def _update_input_text(self):
#         """Update input text when state changes."""
#         value = self._state.value or ""
#         if isinstance(self._inputField._lineEdit, QLineEdit):
#             self._inputField._lineEdit.setText(value)
#         else:
#             self._inputField._lineEdit.setPlainText(value)

#     def _on_input_change(self, value, child=None):
#         """Handle input value change."""
#         self._set_value(value)
#         self.changed.emit(None, child)
#         if self._get_onChange():
#             self._get_onChange()(None, child)

#     def _apply_selected_style(self):
#         """Apply styles when selected."""
#         theme = useTheme()
#         active_border = theme.palette.primary.main
#         hover_background = alpha(theme.palette.primary.main, 0.08)
#         self._inputField.setStyleSheet(f"""
#             QComboBox {{
#                 border-color: {active_border};
#             }}
#         """)

#     def _set_stylesheet(self, component_styled=None):
#         """Set the stylesheet for the Select."""
#         self.theme = useTheme()
#         component_styled = component_styled or self.theme.components
#         select_styles = component_styled.get("Select", {}).get("styles", {})
#         root_styles = select_styles.get("root", {})
#         root_qss = get_qss_style(root_styles)

#         # Handle sx
#         sx = self._get_sx()
#         sx_qss = ""
#         if sx:
#             if isinstance(sx, (list, dict)):
#                 sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
#             elif isinstance(sx, Callable):
#                 sx_result = sx()
#                 if isinstance(sx_result, (list, dict)):
#                     sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
#                 elif isinstance(sx_result, str):
#                     sx_qss = sx_result
#             elif isinstance(sx, str) and sx != "":
#                 sx_qss = sx

#         # Handle classes
#         classes = self._get_classes()
#         classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}") if classes else ""

#         # Handle slotProps
#         slot_props = self._get_slotProps() or {}
#         slot_props_qss = get_qss_style(slot_props.get('root', {}).get('sx', {}), class_name=f"#{self.objectName()}")

#         # Apply MUI classes
#         mui_classes = ["MuiSelect-root"]
#         if self._get_disabled():
#             mui_classes.append("Mui-disabled")

#         stylesheet = f"""
#             #{self.objectName()} {{
#                 {root_qss}
#                 {classes_qss}
#                 {slot_props_qss}
#                 background: transparent;
#             }}
#             {sx_qss}
#         """
#         self.setStyleSheet(stylesheet)

#     def _connect_signals(self):
#         """Connect valueChanged signals of State parameters to their slots."""
#         if isinstance(self._options, State):
#             self._options.valueChanged.connect(self._on_options_changed)
#         if isinstance(self._renderInput, State):
#             self._renderInput.valueChanged.connect(self._on_renderInput_changed)
#         if isinstance(self._autoComplete, State):
#             self._autoComplete.valueChanged.connect(self._on_autoComplete_changed)
#         if isinstance(self._autoHighlight, State):
#             self._autoHighlight.valueChanged.connect(self._on_autoHighlight_changed)
#         if isinstance(self._autoSelect, State):
#             self._autoSelect.valueChanged.connect(self._on_autoSelect_changed)
#         if isinstance(self._blurOnSelect, State):
#             self._blurOnSelect.valueChanged.connect


# ruff: noqa: A003, D100, D101, D102, D103, D104, D105, D107
from __future__ import annotations

import sys
from dataclasses import dataclass, make_dataclass
from typing import Any, Iterable, TypeGuard, TypeVar

from .typing_extensions import dataclass_transform

_T = TypeVar('_T')


@dataclass_transform(kw_only_default=True, frozen_default=True)
def immutable(cls: type[_T]) -> type[_T]:
    if sys.version_info < (3, 10):
        return dataclass(frozen=True)(cls)  # pragma: no cover
    return dataclass(frozen=True, kw_only=True, eq=False, unsafe_hash=True)(cls)


@dataclass_transform(kw_only_default=True, frozen_default=True)
@dataclass(kw_only=True, frozen=True, eq=False, unsafe_hash=True)
class Immutable:
    def __init_subclass__(
        cls: type[Immutable],
        *,
        _immutable_applied: bool = False,
        **kwargs: object,
    ) -> None:
        super().__init_subclass__(**kwargs)
        if not _immutable_applied:
            immutable(cls)

    def __eq__(self: Immutable, other: object) -> bool:
        if not isinstance(other, Immutable):
            return NotImplemented
        return tuple(self.__dict__.values()) == tuple(other.__dict__.values())


class _Immutable(Immutable):
    def __init_subclass__(
        cls: type[_Immutable],
        **kwargs: object,
    ) -> None:
        super().__init_subclass__(_immutable_applied=True, **kwargs)


def is_immutable(obj: object) -> TypeGuard[Immutable]:
    return (
        hasattr(obj, '__dataclass_fields__')
        and hasattr(obj, '__dataclass_params__')
        and getattr(getattr(obj, '__dataclass_params__', None), 'frozen', False)
    )


@dataclass_transform(kw_only_default=True, frozen_default=True)
def make_immutable(
    cls_name: str,
    fields: Iterable[str | tuple[str, Any] | tuple[str, Any, Any]],
) -> type:
    return make_dataclass(
        cls_name,
        fields,
        frozen=True,
        kw_only=True,
        bases=(_Immutable,),
    )



from typing import Dict, TYPE_CHECKING
if TYPE_CHECKING:
    from qtmui.material.styles.create_theme.theme_reducer import ThemeState

from .properties_name import *
from ....system.color_manipulator import alpha


def loading_button(_theme) -> Dict:
    theme: ThemeState = _theme

    return {
        "PyLoadingButton": {
            "styles": {
                "root": lambda ownerState: {
                    "props": {
                        "softVariant": {
                            "loadingIndicatorStart": {
                                "left": 10
                            },
                            "loadingIndicatorEnd": {
                                "right": 14
                            },
                        },
                        "smallSize": {
                            "loadingIndicatorStart": {
                                "left": 10
                            },
                            "loadingIndicatorEnd": {
                                "right": 10
                            },
                        }
                    }
                }
            }
        }
    }



from typing import Callable, Dict, List, Optional, Any
import inspect
from qtmui.hooks import State
from .use_boolean import UseBoolean

class CallbackState:
    """Lớp để lưu trữ trạng thái của callback."""
    def __init__(self):
        self.has_been_called: bool = False
        self.cached_args: List[Any] = []
        self.cached_kwargs: Dict[str, Any] = {}

def useCallback(callback: Callable = None, dependencies: Optional[List[State]] = []) -> Callable:
    """
    Khởi tạo một callback có khả năng cache tham số và gọi lại khi các phụ thuộc thay đổi.
    - callback: Callable - Hàm callback cần được gọi.
    - dependencies: Optional[List[State]] - Danh sách các State phụ thuộc, khi chúng thay đổi, callback sẽ được gọi lại.
    Trả về: Callable - Hàm wrapper để gọi callback với khả năng cache tham số.
    """
    # Kiểm tra số lượng tham số của callback
    sig = inspect.signature(callback)
    # Tạo dictionary chứa các tham số mặc định từ chữ ký của callback
    default_args = {
        k: (v.default if v.default is not inspect.Parameter.empty else None)
        for k, v in sig.parameters.items()
    }

    # Tạo đối tượng để lưu trữ trạng thái của callback
    state = CallbackState()

    # Định nghĩa hàm wrapper để gọi callback và cache tham số
    def wrapper(*args, **kwargs):
        # Cập nhật trạng thái
        state.has_been_called = True
        state.cached_args = list(args)
        state.cached_kwargs = dict(kwargs)
        # Gọi callback với các tham số hiện tại
        return callback(*args, **kwargs)

    # Định nghĩa hàm xử lý khi phụ thuộc thay đổi
    def handle_dependency_change(value, state=state):
        if state.has_been_called:
            # Nếu callback đã được gọi ít nhất một lần, sử dụng tham số đã cache
            callback(*state.cached_args, **state.cached_kwargs)
        else:
            # Nếu chưa được gọi lần nào, gọi callback với các tham số mặc định từ chữ ký
            callback(**default_args)

    # Nếu có các phụ thuộc
    if isinstance(dependencies, list) and len(dependencies):
        for dep in dependencies:
            # Kết nối tín hiệu valueChanged với hàm xử lý
            if isinstance(dep, UseBoolean):
                dep = dep.state
            dep.valueChanged.connect(handle_dependency_change)

    return wrapper


# import uuid
# from typing import Optional, Union, List, Callable, Dict, Any, Type
# from PyQt5.QtWidgets import QFrame, QVBoxLayout, QWidget, QHBoxLayout
# from PyQt5.QtCore import Qt
# from ...base.widget_base import PyWidgetBase
# from qtmui.hooks import State
# from ...components.step import Step
# from ...components.box import Box
# from ...components.divider import Divider
# from qtmui.hooks.use_theme import useTheme
# from qtmui.material.styles.create_theme.components.get_qss_styles import get_qss_style
# from ...utils.validate_params import _validate_param

# class Stepper(QFrame, PyWidgetBase):
#     """
#     A stepper component, styled like Material-UI Stepper.

#     The `Stepper` component displays a sequence of steps, supporting active step selection,
#     horizontal or vertical orientation, and connectors between steps, aligning with MUI Stepper props.
#     Inherits from native component props.

#     Parameters
#     ----------
#     activeStep : State or int, optional
#         Sets the active step (zero-based index). Set to -1 to disable all steps. Default is 0.
#         Can be a `State` object for dynamic updates.
#     alternativeLabel : State or bool, optional
#         If True, step labels are positioned under icons when orientation is 'horizontal'. Default is False.
#         Can be a `State` object for dynamic updates.
#     children : State, List[QWidget], or None, optional
#         Two or more Step components. Default is None.
#         Can be a `State` object for dynamic updates.
#     classes : State or Dict, optional
#         Override or extend styles. Default is None.
#         Can be a `State` object for dynamic updates.
#     component : State, str, Type, or None, optional
#         Component used for the root node. Default is None (uses QFrame).
#         Can be a `State` object for dynamic updates.
#     connector : State, QWidget, or None, optional
#         Element placed between each step. Default is None (uses Divider).
#         Can be a `State` object for dynamic updates.
#     nonLinear : State or bool, optional
#         If True, does not enforce linear step progression. Default is False.
#         Can be a `State` object for dynamic updates.
#     orientation : State or str, optional
#         Component orientation ('horizontal' or 'vertical'). Default is 'horizontal'.
#         Can be a `State` object for dynamic updates.
#     sx : State, List, Dict, Callable, or None, optional
#         System prop for CSS overrides. Default is None.
#         Can be a `State` object for dynamic updates.
#     **kwargs
#         Additional keyword arguments passed to QFrame, supporting native component props.

#     Notes
#     -----
#     - All 9 existing parameters are retained, with `connectorRender` replaced by `connector` to align with MUI Stepper.
#     - Requires at least two Step components in children.
#     - Supports dynamic updates via State objects.
#     - MUI classes applied: `MuiStepper-root`.

#     Demos:
#     - Stepper: https://qtmui.com/material-ui/qtmui-stepper/

#     API Reference:
#     - Stepper API: https://qtmui.com/material-ui/api/stepper/
#     """

#     VALID_ORIENTATIONS = ['horizontal', 'vertical']

#     def __init__(
#         self,
#         activeStep: Union[State, int] = 0,
#         alternativeLabel: Union[State, bool] = False,
#         children: Optional[Union[State, List[QWidget], None]] = None,
#         classes: Optional[Union[State, Dict, None]] = None,
#         component: Optional[Union[State, str, Type, None]] = None,
#         connector: Optional[Union[State, QWidget, None]] = None,
#         nonLinear: Union[State, bool] = False,
#         orientation: Union[State, str] = 'horizontal',
#         sx: Optional[Union[State, List, Dict, Callable, None]] = None,
#         **kwargs
#     ):
#         super().__init__(**kwargs)
#         self.setObjectName(str(uuid.uuid4()))
#         PyWidgetBase._setUpUi(self)
#         self.theme = useTheme()
#         self._widget_references = []

#         # Set properties with validation
#         self._set_activeStep(activeStep)
#         self._set_alternativeLabel(alternativeLabel)
#         self._set_children(children)
#         self._set_classes(classes)
#         self._set_component(component)
#         self._set_connector(connector)
#         self._set_nonLinear(nonLinear)
#         self._set_orientation(orientation)
#         self._set_sx(sx)

#         self._init_ui()
#         self._set_stylesheet()
#         self.useEffect(
            self._set_stylesheet,
            [theme.state]
        )
#         self.destroyed.connect(self._on_destroyed)

#     # Setter and Getter methods
#     @_validate_param(file_path="qtmui.material.stepper", param_name="activeStep", supported_signatures=Union[State, int])
#     def _set_activeStep(self, value):
#         self._activeStep = value
#         if isinstance(value, State):
#             value.valueChanged.connect(self._update_active_step)
#         self._update_active_step()

#     def _get_activeStep(self):
#         active_step = self._activeStep.value if isinstance(self._activeStep, State) else self._activeStep
#         return active_step if active_step >= -1 else 0

#     @_validate_param(file_path="qtmui.material.stepper", param_name="alternativeLabel", supported_signatures=Union[State, bool])
#     def _set_alternativeLabel(self, value):
#         self._alternativeLabel = value
#         if isinstance(value, State):
#             value.valueChanged.connect(self._init_ui)

#     def _get_alternativeLabel(self):
#         return self._alternativeLabel.value if isinstance(self._alternativeLabel, State) else self._alternativeLabel

#     @_validate_param(file_path="qtmui.material.stepper", param_name="children", supported_signatures=Union[State, List[QWidget], type(None)])
#     def _set_children(self, value):
#         self._children = value
#         if isinstance(value, list):
#             if len(value) < 2 or not all(isinstance(child, Step) for child in value):
#                 raise ValueError("Children must contain at least two Step components")
#             self._widget_references.extend(value)
#         if isinstance(value, State):
#             value.valueChanged.connect(self._init_ui)

#     def _get_children(self):
#         return self._children.value if isinstance(self._children, State) else self._children or []

#     @_validate_param(file_path="qtmui.material.stepper", param_name="classes", supported_signatures=Union[State, Dict, type(None)])
#     def _set_classes(self, value):
#         self._classes = value
#         if isinstance(value, State):
#             value.valueChanged.connect(self._set_stylesheet)

#     def _get_classes(self):
#         return self._classes.value if isinstance(self._classes, State) else self._classes

#     @_validate_param(file_path="qtmui.material.stepper", param_name="component", supported_signatures=Union[State, str, Type, type(None)])
#     def _set_component(self, value):
#         self._component = value
#         if isinstance(value, State):
#             value.valueChanged.connect(self._set_stylesheet)

#     def _get_component(self):
#         return self._component.value if isinstance(self._component, State) else self._component

#     @_validate_param(file_path="qtmui.material.stepper", param_name="connector", supported_signatures=Union[State, QWidget, type(None)])
#     def _set_connector(self, value):
#         self._connector = value
#         if isinstance(value, QWidget):
#             self._widget_references.append(value)
#         if isinstance(value, State):
#             value.valueChanged.connect(self._init_ui)

#     def _get_connector(self):
#         return self._connector.value if isinstance(self._connector, State) else self._connector

#     @_validate_param(file_path="qtmui.material.stepper", param_name="nonLinear", supported_signatures=Union[State, bool])
#     def _set_nonLinear(self, value):
#         self._nonLinear = value
#         if isinstance(value, State):
#             value.valueChanged.connect(self._update_active_step)

#     def _get_nonLinear(self):
#         return self._nonLinear.value if isinstance(self._nonLinear, State) else self._nonLinear

#     @_validate_param(file_path="qtmui.material.stepper", param_name="orientation", supported_signatures=Union[State, str], valid_values=VALID_ORIENTATIONS)
#     def _set_orientation(self, value):
#         self._orientation = value
#         if isinstance(value, State):
#             value.valueChanged.connect(self._init_ui)

#     def _get_orientation(self):
#         orientation = self._orientation.value if isinstance(self._orientation, State) else self._orientation
#         return orientation if orientation in self.VALID_ORIENTATIONS else 'horizontal'

#     @_validate_param(file_path="qtmui.material.stepper", param_name="sx", supported_signatures=Union[State, List, Dict, Callable, type(None)])
#     def _set_sx(self, value):
#         self._sx = value
#         if isinstance(value, State):
#             value.valueChanged.connect(self._set_stylesheet)

#     def _get_sx(self):
#         return self._sx.value if isinstance(self._sx, State) else self._sx

#     def _init_ui(self):
#         # Clear existing layout
#         if self.layout():
#             while self.layout().count():
#                 item = self.layout().takeAt(0)
#                 if item.widget():
#                     item.widget().deleteLater()
#                 elif item.layout():
#                     item.layout().deleteLater()

#         # Initialize layout
#         layout_type = QVBoxLayout if self._get_orientation() == 'vertical' else QHBoxLayout
#         self.setLayout(layout_type())
#         self.layout().setContentsMargins(0, 0, 0, 0)
#         self.layout().setSpacing(self.theme.spacing(2))

#         progress_layout = layout_type()
#         progress_layout.setSpacing(self.theme.spacing(2))
#         self.layout().addLayout(progress_layout)

#         # Add steps and connectors
#         children = self._get_children()
#         connector = self._get_connector() or Divider()

#         for index, step in enumerate(children):
#             step.setParent(self)
#             step.set_index(index)
#             step.set_last(index == len(children) - 1)
#             progress_layout.addWidget(step)
#             if index < len(children) - 1:
#                 connector_instance = connector() if callable(connector) else connector
#                 connector_instance.setParent(self)
#                 progress_layout.addWidget(connector_instance)
#                 self._widget_references.append(connector_instance)

#         # Add labels for alternativeLabel in horizontal orientation
#         if self._get_alternativeLabel() and self._get_orientation() == 'horizontal':
#             label_layout = QHBoxLayout()
#             label_layout.setSpacing(self.theme.spacing(2))
#             self.layout().addLayout(label_layout)
#             for index, step in enumerate(children):
#                 label = step._get_label()
#                 if label:
#                     label.setParent(self)
#                     label_layout.addWidget(label)
#                 if index < len(children) - 1:
#                     spacer = Divider(color="transparent")
#                     spacer.setParent(self)
#                     label_layout.addWidget(spacer)
#                     self._widget_references.append(spacer)

#         self._update_active_step()

#     def _set_stylesheet(self):
#         component_styled = self.theme.components
#         stepper_styles = component_styled.get("Stepper", {}).get("styles", {})
#         root_styles = stepper_styles.get("root", {})
#         root_qss = get_qss_style(root_styles)

#         # Handle classes
#         classes = self._get_classes() or {}
#         classes_qss = get_qss_style(classes, class_name=f"#{self.objectName()}")

#         # Handle sx
#         sx = self._get_sx()
#         sx_qss = ""
#         if sx:
#             if isinstance(sx, (list, dict)):
#                 sx_qss = get_qss_style(sx, class_name=f"#{self.objectName()}")
#             elif isinstance(sx, Callable):
#                 sx_result = sx()
#                 if isinstance(sx_result, (list, dict)):
#                     sx_qss = get_qss_style(sx_result, class_name=f"#{self.objectName()}")
#                 elif isinstance(sx_result, str):
#                     sx_qss = sx_result
#             elif isinstance(sx, str) and sx != "":
#                 sx_qss = sx

#         stylesheet = f"""
#             #{self.objectName()} {{
#                 {root_qss}
#                 {classes_qss}
#                 background: transparent;
#                 padding: {self.theme.spacing(2)}px;
#             }}
#             {sx_qss}
#         """
#         self.setStyleSheet(stylesheet)

#     def _update_active_step(self):
#         active_step = self._get_activeStep()
#         non_linear = self._get_nonLinear()
#         children = self._get_children()

#         for index, step in enumerate(children):
#             if not isinstance(step, Step):
#                 continue
#             if active_step == -1:
#                 step.set_active(False)
#                 step.set_completed(False)
#                 step._set_icon_widget("inactive")
#             elif non_linear:
#                 step.set_active(index == active_step)
#                 step.set_completed(step._get_completed())
#                 step._set_icon_widget("current" if index == active_step else "normal")
#             else:
#                 step.set_active(index == active_step)
#                 step.set_completed(index < active_step)
#                 if index < active_step:
#                     step._set_icon_widget("complete")
#                 elif index == active_step:
#                     step._set_icon_widget("current")
#                 else:
#                     step._set_icon_widget("normal")

#     def _on_destroyed(self):
#         self._widget_references.clear()

#     def generate_stylesheet(self):
#         return self.styleSheet()

#     # Public setters
#     def set_active_step(self, step: int):
#         self._set_activeStep(step)
#         self._update_active_step()

#     def set_orientation(self, orientation: str):
#         self._set_orientation(orientation)
#         self._init_ui()

#     def set_connector(self, connector: QWidget):
#         self._set_connector(connector)
#         self._init_ui()

#     # Public getters
#     def get_active_step(self):
#         return self._get_activeStep()

#     def update_ui(self):
#         self._init_ui()
#         self._set_stylesheet()


from PySide6.QtWidgets import QPushButton

class DelegateButton(QPushButton):
    def __init__(self, parent=None):
        super(DelegateButton, self).__init__(parent)

        # self.setLayout(QHBoxLayout())
        size = 50
        self.setFixedSize(size, size)
        self.setStyleSheet("""
            QPushButton{
                background-color: rgb(237, 237, 237);
                height: 30px;
                border: 0px solid transparent;
                border-radius: 4px;
                margin-left: 2px;
                color: #6b6b6b;
            }
            QPushButton:hover {
                background-color: rgb(227, 227, 227);
            }
            """)
